@@ -276,6 +276,29 @@ static void emit_event(pa_context *c, struct global *g, pa_subscription_event_ty
 	}
 }
 
+static struct param *add_param(struct spa_list *params, uint32_t id, const struct spa_pod *param)
+{
+	struct param *p;
+
+	if (param == NULL || !spa_pod_is_object(param)) {
+		errno = EINVAL;
+		return NULL;
+	}
+	if (id == SPA_ID_INVALID)
+		id = SPA_POD_OBJECT_ID(param);
+
+	p = malloc(sizeof(struct param) + SPA_POD_SIZE(param));
+	if (p == NULL)
+		return NULL;
+
+	p->id = id;
+	p->param = SPA_MEMBER(p, sizeof(struct param), struct spa_pod);
+	memcpy(p->param, param, SPA_POD_SIZE(param));
+	spa_list_append(params, &p->link);
+
+	return p;
+}
+
 static void remove_params(struct spa_list *params, uint32_t id)
 {
 	struct param *p, *t;
@@ -325,39 +348,48 @@ static void device_event_info(void *object, const struct pw_device_info *info)
 	}
 	if (info->change_mask & PW_DEVICE_CHANGE_MASK_PARAMS) {
 		for (n = 0; n < info->n_params; n++) {
-			if (!(info->params[n].flags & SPA_PARAM_INFO_READ))
+			uint32_t id = info->params[n].id;
+			bool do_enum = true;
+
+			if (info->params[n].user == 0)
 				continue;
 
-			switch (info->params[n].id) {
+			info->params[n].user = 0;
+
+			switch (id) {
 			case SPA_PARAM_EnumProfile:
 				if (g->card_info.pending_profiles)
 					continue;
-				remove_params(&g->card_info.profiles, SPA_PARAM_EnumProfile);
+				remove_params(&g->card_info.profiles, id);
 				g->card_info.n_profiles = 0;
-				g->card_info.pending_profiles = true;
-				pw_device_enum_params((struct pw_device*)g->proxy,
-					0, SPA_PARAM_EnumProfile, 0, -1, NULL);
-				break;
-			case SPA_PARAM_Profile:
-				pw_device_enum_params((struct pw_device*)g->proxy,
-					0, SPA_PARAM_Profile, 0, -1, NULL);
 				break;
 			case SPA_PARAM_EnumRoute:
 				if (g->card_info.pending_ports)
 					continue;
-				remove_params(&g->card_info.ports, SPA_PARAM_EnumRoute);
+				remove_params(&g->card_info.ports, id);
 				g->card_info.n_ports = 0;
-				g->card_info.pending_ports = true;
-				pw_device_enum_params((struct pw_device*)g->proxy,
-					0, SPA_PARAM_EnumRoute, 0, -1, NULL);
 				break;
 			case SPA_PARAM_Route:
-				pw_device_enum_params((struct pw_device*)g->proxy,
-					0, SPA_PARAM_Route, 0, -1, NULL);
+			case SPA_PARAM_Profile:
 				break;
 			default:
+				do_enum = false;
 				break;
 			}
+			if (!(info->params[n].flags & SPA_PARAM_INFO_READ))
+				continue;
+			if (do_enum) {
+				switch (id) {
+				case SPA_PARAM_EnumProfile:
+					g->card_info.pending_profiles = true;
+					break;
+				case SPA_PARAM_EnumRoute:
+					g->card_info.pending_ports = true;
+					break;
+				}
+				pw_device_enum_params((struct pw_device*)g->proxy,
+						0, id, 0, -1, NULL);
+			}
 		}
 	}
 	global_sync(g);
@@ -445,7 +477,6 @@ static void device_event_param(void *object, int seq,
 	{
 		uint32_t index;
 		const char *name;
-		struct param *p;
 
 		if (spa_pod_parse_object(param,
 				SPA_TYPE_OBJECT_ParamProfile, NULL,
@@ -454,15 +485,9 @@ static void device_event_param(void *object, int seq,
 			pw_log_warn("device %d: can't parse profile", g->id);
 			return;
 		}
-		p = malloc(sizeof(struct param) + SPA_POD_SIZE(param));
-		if (p) {
-			p->id = id;
-			p->seq = seq;
-			p->param = SPA_MEMBER(p, sizeof(struct param), struct spa_pod);
-			memcpy(p->param, param, SPA_POD_SIZE(param));
-			spa_list_append(&g->card_info.profiles, &p->link);
+		if (add_param(&g->card_info.profiles, id, param))
 			g->card_info.n_profiles++;
-		}
+
 		pw_log_debug("device %d: enum profile %d: \"%s\" n_profiles:%d", g->id,
 				index, name, g->card_info.n_profiles);
 		break;
@@ -484,7 +509,6 @@ static void device_event_param(void *object, int seq,
 	{
 		uint32_t index;
 		const char *name;
-		struct param *p;
 
 		if (spa_pod_parse_object(param,
 				SPA_TYPE_OBJECT_ParamRoute, NULL,
@@ -493,15 +517,9 @@ static void device_event_param(void *object, int seq,
 			pw_log_warn("device %d: can't parse route", g->id);
 			return;
 		}
-		p = malloc(sizeof(struct param) + SPA_POD_SIZE(param));
-		if (p) {
-			p->id = id;
-			p->seq = seq;
-			p->param = SPA_MEMBER(p, sizeof(struct param), struct spa_pod);
-			memcpy(p->param, param, SPA_POD_SIZE(param));
-			spa_list_append(&g->card_info.ports, &p->link);
+		if (add_param(&g->card_info.ports, id, param))
 			g->card_info.n_ports++;
-		}
+
 		pw_log_debug("device %d: enum route %d: \"%s\"", g->id, index, name);
 		break;
 	}
@@ -705,7 +723,7 @@ static void device_sync_ports(struct global *g)
 		while (info) {
 			struct spa_pod_parser prs;
 			struct spa_pod_frame f[1];
-			uint32_t n, n_items;
+			int32_t n, n_items;
 			const char *key, *value;
 
 			spa_pod_parser_pod(&prs, info);
@@ -806,24 +824,25 @@ static void node_event_info(void *object, const struct pw_node_info *info)
 	else
 		g->node_info.profile_device_id = SPA_ID_INVALID;
 
-	if (info->change_mask & PW_NODE_CHANGE_MASK_PARAMS && !g->subscribed) {
-		uint32_t subscribed[32], n_subscribed = 0;
-
+	if (info->change_mask & PW_NODE_CHANGE_MASK_PARAMS) {
 		for (i = 0; i < info->n_params; i++) {
+			if (info->params[i].user == 0)
+				continue;
+			info->params[i].user = 0;
+
+			if (!(info->params[i].flags & SPA_PARAM_INFO_READ))
+				continue;
+
 			switch (info->params[i].id) {
-			case SPA_PARAM_EnumFormat:
 			case SPA_PARAM_Props:
-				subscribed[n_subscribed++] = info->params[i].id;
+			case SPA_PARAM_Format:
+				pw_device_enum_params((struct pw_device*)g->proxy,
+					0, info->params[i].id, 0, -1, NULL);
 				break;
 			default:
 				break;
 			}
 		}
-		if (n_subscribed > 0) {
-			pw_node_subscribe_params((struct pw_node*)g->proxy,
-					subscribed, n_subscribed);
-			g->subscribed = true;
-		}
 	}
 	global_sync(g);
 }
@@ -840,6 +859,9 @@ static void node_event_param(void *object, int seq,
 		if (!SPA_FLAG_IS_SET(g->node_info.flags, NODE_FLAG_DEVICE_VOLUME | NODE_FLAG_DEVICE_MUTE))
 			parse_props(g, param, false);
 		break;
+	case SPA_PARAM_Format:
+		pa_format_parse_param(param, &g->node_info.sample_spec, &g->node_info.channel_map);
+		break;
 	default:
 		break;
 	}
@@ -962,18 +984,22 @@ static int metadata_property(void *object,
                         const char *value)
 {
 	struct global *global = object;
-	int res;
+	pa_context *c = global->context;
+	uint32_t val;
+	bool changed = false;
 
-	if ((res = pa_metadata_update(global, subject, key, type, value)) < 0)
-		return res;
-
-	if (key) {
-		if (strcmp(key, METADATA_DEFAULT_SINK) == 0 ||
-		    strcmp(key, METADATA_DEFAULT_SOURCE) == 0) {
-			emit_event(global->context, global, PA_SUBSCRIPTION_EVENT_CHANGE);
-		}
+	if (key && strcmp(key, METADATA_DEFAULT_SINK) == 0) {
+		val = value ? (uint32_t)atoi(value) : SPA_ID_INVALID;
+		changed = c->default_sink != val;
+		c->default_sink = val;
+	} else if (key && strcmp(key, METADATA_DEFAULT_SOURCE) == 0) {
+		val = value ? (uint32_t)atoi(value) : SPA_ID_INVALID;
+		changed = c->default_source != val;
 	}
-	return res;
+	if (changed)
+		emit_event(global->context, global, PA_SUBSCRIPTION_EVENT_CHANGE);
+
+	return 0;
 }
 
 static const struct pw_metadata_events metadata_events = {
@@ -1103,6 +1129,8 @@ static int set_mask(pa_context *c, struct global *g)
 			g->node_info.device_id = SPA_ID_INVALID;
 
 		ginfo = &node_info;
+		g->node_info.sample_spec.format = PA_SAMPLE_S16NE;
+		g->node_info.sample_spec.rate = 44100;
 		g->node_info.volume = 1.0f;
 		g->node_info.mute = false;
 		g->node_info.base_volume = 1.0f;
@@ -1389,6 +1417,8 @@ pa_context *pa_context_new_with_proplist(pa_mainloop_api *mainloop, const char *
 	c->proplist = p ? pa_proplist_copy(p) : pa_proplist_new();
 	c->refcount = 1;
 	c->client_index = PA_INVALID_INDEX;
+	c->default_sink = SPA_ID_INVALID;
+	c->default_source = SPA_ID_INVALID;
 
 	if (name)
 		pa_proplist_sets(c->proplist, PA_PROP_APPLICATION_NAME, name);
@@ -1621,6 +1651,9 @@ static void do_default_node(pa_operation *o, void *userdata)
 	g = pa_context_find_global_by_name(c, d->mask, d->name);
 	if (g == NULL) {
 		error = PA_ERR_NOENTITY;
+	} else if (!SPA_FLAG_IS_SET(g->permissions, PW_PERM_M) ||
+		(c->metadata && !SPA_FLAG_IS_SET(c->metadata->permissions, PW_PERM_W|PW_PERM_X))) {
+		error = PA_ERR_ACCESS;
 	} else if (c->metadata) {
 		char buf[16];
 		snprintf(buf, sizeof(buf), "%d", g->id);
