@@ -104,13 +104,15 @@ static int sink_callback(pa_context *c, struct global *g, struct sink_data *d)
 	else
 		i.description = "Unknown";
 
-	i.sample_spec.format = PA_SAMPLE_S16LE;
-	i.sample_spec.rate = 44100;
+	i.sample_spec = g->node_info.sample_spec;
 	if (g->node_info.n_channel_volumes)
 		i.sample_spec.channels = g->node_info.n_channel_volumes;
 	else
 		i.sample_spec.channels = 2;
-	pa_channel_map_init_auto(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
+	if (i.sample_spec.channels == g->node_info.channel_map.channels)
+		i.channel_map = g->node_info.channel_map;
+	else
+		pa_channel_map_init_auto(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
 	i.owner_module = 0;
 	i.volume.channels = i.sample_spec.channels;
 	for (n = 0; n < i.volume.channels; n++)
@@ -843,13 +845,15 @@ static int source_callback(pa_context *c, struct global *g, struct source_data *
 		i.description = str;
 	else
 		i.description = "unknown";
-	i.sample_spec.format = PA_SAMPLE_S16LE;
-	i.sample_spec.rate = 44100;
+	i.sample_spec = g->node_info.sample_spec;
 	if (g->node_info.n_channel_volumes)
 		i.sample_spec.channels = g->node_info.n_channel_volumes;
 	else
 		i.sample_spec.channels = 2;
-	pa_channel_map_init_auto(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
+	if (i.sample_spec.channels == g->node_info.channel_map.channels)
+		i.channel_map = g->node_info.channel_map;
+	else
+		pa_channel_map_init_auto(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
 	i.owner_module = 0;
 	i.volume.channels = i.sample_spec.channels;
 	for (n = 0; n < i.volume.channels; n++)
@@ -1244,25 +1248,23 @@ struct server_data {
 static const char *get_default_name(pa_context *c, uint32_t mask)
 {
 	struct global *g;
-	const char *str, *id = NULL, *type, *key;
+	const char *str;
+	uint32_t id = SPA_ID_INVALID;
 
 	if (c->metadata) {
 		if (mask & PA_SUBSCRIPTION_MASK_SINK)
-			key = METADATA_DEFAULT_SINK;
+			id = c->default_sink;
 		else if (mask & PA_SUBSCRIPTION_MASK_SOURCE)
-			key = METADATA_DEFAULT_SOURCE;
+			id = c->default_source;
 		else
 			return NULL;
-
-		if (pa_metadata_get(c->metadata, PW_ID_CORE, key, &type, &id) <= 0)
-			id = NULL;
 	}
 	spa_list_for_each(g, &c->globals, link) {
 		if ((g->mask & mask) != mask)
 			continue;
 		if (g->props != NULL &&
 		    (str = pw_properties_get(g->props, PW_KEY_NODE_NAME)) != NULL &&
-		    (id == NULL || (uint32_t)atoi(id) == g->id))
+		    (id == SPA_ID_INVALID || id == g->id))
 			return str;
 	}
 	return "unknown";
@@ -1913,12 +1915,15 @@ static int sink_input_callback(pa_context *c, struct sink_input_data *d, struct
 		i.format = s->format;
 	}
 	else {
-		i.sample_spec.format = PA_SAMPLE_S16LE;
-		i.sample_spec.rate = 44100;
-		i.sample_spec.channels = g->node_info.n_channel_volumes;
-		if (i.sample_spec.channels == 0)
+		i.sample_spec = g->node_info.sample_spec;
+		if (g->node_info.n_channel_volumes)
+			i.sample_spec.channels = g->node_info.n_channel_volumes;
+		else
 			i.sample_spec.channels = 2;
-		pa_channel_map_init_auto(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
+		if (i.sample_spec.channels == g->node_info.channel_map.channels)
+			i.channel_map = g->node_info.channel_map;
+		else
+			pa_channel_map_init_auto(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
 		ii[0].encoding = PA_ENCODING_PCM;
 		ii[0].plist = pa_proplist_new();
 		i.format = ii;
@@ -2049,7 +2054,7 @@ static void do_target_node(pa_operation *o, void *userdata)
 {
 	struct target_node *d = userdata;
 	pa_context *c = o->context;
-	struct global *g;
+	struct global *g, *t;
 	int error = 0;
 
 	pw_log_debug("%p", c);
@@ -2061,19 +2066,22 @@ static void do_target_node(pa_operation *o, void *userdata)
 	}
 
 	if (d->target_name) {
-		g = pa_context_find_global_by_name(c, d->target_mask, d->target_name);
+		t = pa_context_find_global_by_name(c, d->target_mask, d->target_name);
 	} else {
-		if ((g = pa_context_find_global(c, d->target_idx)) == NULL ||
-		    !(g->mask & d->target_mask))
-			g = NULL;
+		if ((t = pa_context_find_global(c, d->target_idx)) == NULL ||
+		    !(t->mask & d->target_mask))
+			t = NULL;
 	}
-	if (g == NULL) {
+	if (t == NULL) {
 		error = PA_ERR_NOENTITY;
+	} else if (!SPA_FLAG_IS_SET(g->permissions, PW_PERM_M) ||
+		(c->metadata && !SPA_FLAG_IS_SET(c->metadata->permissions, PW_PERM_W|PW_PERM_X))) {
+		error = PA_ERR_ACCESS;
 	} else if (c->metadata) {
 		char buf[16];
-		snprintf(buf, sizeof(buf), "%d", g->id);
+		snprintf(buf, sizeof(buf), "%d", t->id);
 		pw_metadata_set_property(c->metadata->proxy,
-				d->idx, d->key, SPA_TYPE_INFO_BASE "Id", buf);
+				g->id, d->key, SPA_TYPE_INFO_BASE "Id", buf);
 	} else {
 		error = PA_ERR_NOTIMPLEMENTED;
 	}
@@ -2324,12 +2332,15 @@ static int source_output_callback(struct source_output_data *d, pa_context *c, s
 		i.format = s->format;
 	}
 	else {
-		i.sample_spec.format = PA_SAMPLE_S16LE;
-		i.sample_spec.rate = 44100;
-		i.sample_spec.channels = g->node_info.n_channel_volumes;
-		if (i.sample_spec.channels == 0)
+		i.sample_spec = g->node_info.sample_spec;
+		if (g->node_info.n_channel_volumes)
+			i.sample_spec.channels = g->node_info.n_channel_volumes;
+		else
 			i.sample_spec.channels = 2;
-		pa_channel_map_init_auto(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
+		if (i.sample_spec.channels == g->node_info.channel_map.channels)
+			i.channel_map = g->node_info.channel_map;
+		else
+			pa_channel_map_init_auto(&i.channel_map, i.sample_spec.channels, PA_CHANNEL_MAP_OSS);
 		ii[0].encoding = PA_ENCODING_PCM;
 		ii[0].plist = pa_proplist_new();
 		i.format = ii;
