@@ -317,7 +317,7 @@ static int report_jack_state(snd_mixer_elem_t *melem, unsigned int mask)
     pa_log_debug("Jack '%s' is now %s", pa_strnull(snd_hctl_elem_get_name(elem)),
 		    plugged_in ? "plugged in" : "unplugged");
 
-    size = sizeof(struct temp_port_avail) * pa_hashmap_size(impl->jacks)+1;
+    size = sizeof(struct temp_port_avail) * (pa_hashmap_size(impl->jacks)+1);
     tports = tp = alloca(size);
     memset(tports, 0, size);
 
@@ -1111,13 +1111,13 @@ struct acp_card *acp_card_new(uint32_t index, const struct acp_dict *props)
 	impl->use_ucm = true;
 
 	if (props) {
-		if ((s = acp_dict_lookup(props, "use-ucm")) != NULL)
+		if ((s = acp_dict_lookup(props, "api.alsa.use-ucm")) != NULL)
 			impl->use_ucm = (strcmp(s, "true") == 0 || atoi(s) == 1);
-		if ((s = acp_dict_lookup(props, "profile-set")) != NULL)
-			profile_set = s;
-		if ((s = acp_dict_lookup(props, "ignore-dB")) != NULL)
+		if ((s = acp_dict_lookup(props, "api.alsa.ignore-dB")) != NULL)
 			ignore_dB = (strcmp(s, "true") == 0 || atoi(s) == 1);
-		if ((s = acp_dict_lookup(props, "profile")) != NULL)
+		if ((s = acp_dict_lookup(props, "device.profile-set")) != NULL)
+			profile_set = s;
+		if ((s = acp_dict_lookup(props, "device.profile")) != NULL)
 			profile = s;
 	}
 
