@@ -99,13 +99,20 @@ static void handle_acp_poll(struct spa_source *source)
 		this->sources[i].rmask = 0;
 }
 
+static void remove_sources(struct impl *this)
+{
+	int i;
+	for (i = 0; i < this->n_pfds; i++) {
+		spa_loop_remove_source(this->loop, &this->sources[i]);
+	}
+	this->n_pfds = 0;
+}
+
 static int setup_sources(struct impl *this)
 {
 	int i;
 
-	for (i = 0; i < this->n_pfds; i++) {
-		spa_loop_remove_source(this->loop, &this->sources[i]);
-	}
+	remove_sources(this);
 
 	this->n_pfds = acp_card_poll_descriptors(this->card, this->pfds, MAX_POLL);
 
@@ -774,6 +781,8 @@ static SPA_PRINTF_FUNC(6,0) void impl_acp_log_func(void *data,
 
 static int impl_clear(struct spa_handle *handle)
 {
+	struct impl *this = (struct impl *) handle;
+	remove_sources(this);
 	return 0;
 }
 
@@ -793,7 +802,8 @@ impl_init(const struct spa_handle_factory *factory,
 {
 	struct impl *this;
 	const char *str;
-	struct acp_dict_item items[4];
+	struct acp_dict_item *items = NULL;
+	const struct spa_dict_item *it;
 	uint32_t n_items = 0;
 
 	spa_return_val_if_fail(factory != NULL, -EINVAL);
@@ -825,10 +835,10 @@ impl_init(const struct spa_handle_factory *factory,
 	if (info) {
 		if ((str = spa_dict_lookup(info, SPA_KEY_API_ALSA_PATH)) != NULL)
 			snprintf(this->props.device, sizeof(this->props.device)-1, "%s", str);
-		if ((str = spa_dict_lookup(info, SPA_KEY_DEVICE_PROFILE_SET)) != NULL)
-			items[n_items++] = ACP_DICT_ITEM_INIT("profile-set", str);
-		if ((str = spa_dict_lookup(info, SPA_KEY_DEVICE_PROFILE)) != NULL)
-			items[n_items++] = ACP_DICT_ITEM_INIT("profile", str);
+
+		items = alloca((info->n_items) * sizeof(*items));
+		spa_dict_for_each(it, info)
+			items[n_items++] = ACP_DICT_ITEM_INIT(it->key, it->value);
 	}
 
 	spa_log_debug(this->log, "probe card %s", this->props.device);
