@@ -85,11 +85,6 @@ struct node_data {
 	struct spa_hook client_node_listener;
 	struct spa_hook proxy_client_node_listener;
 
-	struct pw_proxy *proxy;
-	struct spa_hook proxy_listener;
-
-	struct spa_io_position *position;
-
 	struct spa_list links;
 };
 
@@ -127,10 +122,11 @@ do_deactivate_link(struct spa_loop *loop,
 
 static void clear_link(struct node_data *data, struct link *link)
 {
-	pw_loop_invoke(data->context->data_loop,
+	struct pw_context *context = data->context;
+	pw_loop_invoke(context->data_loop,
 		do_deactivate_link, SPA_ID_INVALID, NULL, 0, true, link);
 	pw_memmap_free(link->map);
-	close(link->signalfd);
+	spa_system_close(context->data_system, link->signalfd);
 	spa_list_remove(&link->link);
 	free(link);
 }
@@ -153,7 +149,7 @@ static void clean_transport(struct node_data *data)
 	pw_memmap_free(data->activation);
 	data->node->rt.activation = data->node->activation->map->ptr;
 
-	close(data->rtwritefd);
+	spa_system_close(data->context->data_system, data->rtwritefd);
 	data->have_transport = false;
 }
 
@@ -252,15 +248,15 @@ static struct mix *ensure_mix(struct node_data *data,
 static int client_node_transport(void *object,
 			int readfd, int writefd, uint32_t mem_id, uint32_t offset, uint32_t size)
 {
-	struct pw_proxy *proxy = object;
-	struct node_data *data = proxy->user_data;
+	struct node_data *data = object;
+	struct pw_proxy *proxy = (struct pw_proxy*)data->client_node;
 
 	clean_transport(data);
 
 	data->activation = pw_mempool_map_id(data->pool, mem_id,
 				PW_MEMMAP_FLAG_READWRITE, offset, size, NULL);
 	if (data->activation == NULL) {
-		pw_log_debug("remote-node %p: can't map activation: %m", proxy);
+		pw_log_warn("remote-node %p: can't map activation: %m", proxy);
 		return -errno;
 	}
 
@@ -269,8 +265,8 @@ static int client_node_transport(void *object,
 	pw_log_debug("remote-node %p: fds:%d %d node:%u activation:%p",
 		proxy, readfd, writefd, data->remote_id, data->activation->ptr);
 
-        data->rtwritefd = writefd;
-	close(data->node->source.fd);
+	data->rtwritefd = writefd;
+	spa_system_close(data->context->data_system, data->node->source.fd);
 	data->node->source.fd = readfd;
 
 	data->have_transport = true;
@@ -281,9 +277,8 @@ static int client_node_transport(void *object,
 	return 0;
 }
 
-static int add_node_update(struct pw_proxy *proxy, uint32_t change_mask)
+static int add_node_update(struct node_data *data, uint32_t change_mask)
 {
-	struct node_data *data = proxy->user_data;
 	struct pw_impl_node *node = data->node;
 	struct spa_node_info ni = SPA_NODE_INFO_INIT();
 	uint32_t n_params = 0;
@@ -341,9 +336,8 @@ static int add_node_update(struct pw_proxy *proxy, uint32_t change_mask)
 	return res;
 }
 
-static int add_port_update(struct pw_proxy *proxy, struct pw_impl_port *port, uint32_t change_mask)
+static int add_port_update(struct node_data *data, struct pw_impl_port *port, uint32_t change_mask)
 {
-	struct node_data *data = proxy->user_data;
 	struct spa_port_info pi = SPA_PORT_INFO_INIT();
 	uint32_t n_params = 0;
 	struct spa_pod **params = NULL;
@@ -406,8 +400,7 @@ static int
 client_node_set_param(void *object, uint32_t id, uint32_t flags,
 		      const struct spa_pod *param)
 {
-	struct pw_proxy *proxy = object;
-	struct node_data *data = proxy->user_data;
+	struct node_data *data = object;
 	return spa_node_set_param(data->node->node, id, flags, param);
 }
 
@@ -418,25 +411,25 @@ client_node_set_io(void *object,
 		   uint32_t offset,
 		   uint32_t size)
 {
-	struct pw_proxy *proxy = object;
-	struct node_data *data = proxy->user_data;
-	struct pw_memmap *mm;
+	struct node_data *data = object;
+	struct pw_proxy *proxy = (struct pw_proxy*)data->client_node;
+	struct pw_memmap *old, *mm;
 	void *ptr;
 	uint32_t tag[5] = { data->remote_id, id, };
+	int res;
 
-	if ((mm = pw_mempool_find_tag(data->pool, tag, sizeof(tag))) != NULL)
-		pw_memmap_free(mm);
+	old = pw_mempool_find_tag(data->pool, tag, sizeof(tag));
 
 	if (memid == SPA_ID_INVALID) {
 		mm = ptr = NULL;
 		size = 0;
-	}
-	else {
+	} else {
 		mm = pw_mempool_map_id(data->pool, memid,
 				PW_MEMMAP_FLAG_READWRITE, offset, size, tag);
 		if (mm == NULL) {
 			pw_log_warn("can't map memory id %u: %m", memid);
-			return -errno;
+			res = -errno;
+			goto exit;
 		}
 		ptr = mm->ptr;
 	}
@@ -444,15 +437,16 @@ client_node_set_io(void *object,
 	pw_log_debug("node %p: set io %s %p", proxy,
 			spa_debug_type_find_name(spa_type_io, id), ptr);
 
-	switch (id) {
-	case SPA_IO_Position:
-		data->position = ptr;
-		break;
-	default:
-		break;
-	}
+	res =  spa_node_set_io(data->node->node, id, ptr, size);
 
-	return spa_node_set_io(data->node->node, id, ptr, size);
+	if (old != NULL)
+		pw_memmap_free(old);
+exit:
+	if (res < 0) {
+		pw_log_error("node %p: set_io: %s", proxy, spa_strerror(res));
+		pw_proxy_errorf(proxy, res, "node_set_io failed: %s", spa_strerror(res));
+	}
+	return res;
 }
 
 static int client_node_event(void *object, const struct spa_event *event)
@@ -463,8 +457,8 @@ static int client_node_event(void *object, const struct spa_event *event)
 
 static int client_node_command(void *object, const struct spa_command *command)
 {
-	struct pw_proxy *proxy = object;
-	struct node_data *data = proxy->user_data;
+	struct node_data *data = object;
+	struct pw_proxy *proxy = (struct pw_proxy*)data->client_node;
 	int res;
 
 	switch (SPA_NODE_COMMAND_ID(command)) {
@@ -506,7 +500,8 @@ static int
 client_node_add_port(void *object, enum spa_direction direction, uint32_t port_id,
 		const struct spa_dict *props)
 {
-	struct pw_proxy *proxy = object;
+	struct node_data *data = object;
+	struct pw_proxy *proxy = (struct pw_proxy*)data->client_node;
 	pw_log_warn("add port not supported");
 	pw_proxy_error(proxy, -ENOTSUP, "add port not supported");
 	return -ENOTSUP;
@@ -515,7 +510,8 @@ client_node_add_port(void *object, enum spa_direction direction, uint32_t port_i
 static int
 client_node_remove_port(void *object, enum spa_direction direction, uint32_t port_id)
 {
-	struct pw_proxy *proxy = object;
+	struct node_data *data = object;
+	struct pw_proxy *proxy = (struct pw_proxy*)data->client_node;
 	pw_log_warn("remove port not supported");
 	pw_proxy_error(proxy, -ENOTSUP, "remove port not supported");
 	return -ENOTSUP;
@@ -552,8 +548,8 @@ client_node_port_set_param(void *object,
 			   uint32_t id, uint32_t flags,
 			   const struct spa_pod *param)
 {
-	struct pw_proxy *proxy = object;
-	struct node_data *data = proxy->user_data;
+	struct node_data *data = object;
+	struct pw_proxy *proxy = (struct pw_proxy*)data->client_node;
 	struct pw_impl_port *port;
 	int res;
 
@@ -563,9 +559,13 @@ client_node_port_set_param(void *object,
 		goto error_exit;
 	}
 
-        pw_log_debug("port %p: set param %d %p", port, id, param);
+	pw_log_debug("port %p: set param %d %p", port, id, param);
 
-        if (id == SPA_PARAM_Format) {
+	res = pw_impl_port_set_param(port, id, flags, param);
+	if (res < 0)
+		goto error_exit;
+
+	if (id == SPA_PARAM_Format) {
 		struct mix *mix;
 		spa_list_for_each(mix, &data->mix[direction], link) {
 			if (mix->port->port_id == port_id)
@@ -573,14 +573,10 @@ client_node_port_set_param(void *object,
 		}
 	}
 
-	res = pw_impl_port_set_param(port, id, flags, param);
-	if (res < 0)
-		goto error_exit;
-
 	return res;
 
 error_exit:
-        pw_log_error("port %p: set_param %d %p: %s", port, id, param, spa_strerror(res));
+	pw_log_error("port %p: set_param %d %p: %s", port, id, param, spa_strerror(res));
 	pw_proxy_errorf(proxy, res, "port_set_param: %s", spa_strerror(res));
 	return res;
 }
@@ -591,8 +587,8 @@ client_node_port_use_buffers(void *object,
 			     uint32_t flags,
 			     uint32_t n_buffers, struct pw_client_node_buffer *buffers)
 {
-	struct pw_proxy *proxy = object;
-	struct node_data *data = proxy->user_data;
+	struct node_data *data = object;
+	struct pw_proxy *proxy = (struct pw_proxy*)data->client_node;
 	struct buffer *bid;
 	uint32_t i, j;
 	struct spa_buffer *b, **bufs;
@@ -657,7 +653,7 @@ client_node_port_use_buffers(void *object,
 		b->datas = SPA_MEMBER(b->metas, sizeof(struct spa_meta) * b->n_metas,
 				       struct spa_data);
 
-		pw_log_debug("add buffer %d %d %u %u %p", mm->block->id,
+		pw_log_debug("add buffer mem:%d id:%d offset:%u size:%u %p", mm->block->id,
 				bid->id, buffers[i].offset, buffers[i].size, bid->buf);
 
 		offset = 0;
@@ -700,8 +696,8 @@ client_node_port_use_buffers(void *object,
 				int offs = SPA_PTR_TO_INT(d->data);
 				d->data = SPA_MEMBER(mm->ptr, offs, void);
 				d->fd = -1;
-				pw_log_debug(" data %d %u -> mem %p maxsize %d",
-						j, bid->id, d->data, d->maxsize);
+				pw_log_debug(" data %d id:%u -> mem:%p offs:%d maxsize:%d",
+						j, bid->id, d->data, offs, d->maxsize);
 			} else {
 				pw_log_warn("unknown buffer data type %d", d->type);
 			}
@@ -738,10 +734,10 @@ client_node_port_set_io(void *object,
                         uint32_t offset,
                         uint32_t size)
 {
-	struct pw_proxy *proxy = object;
-	struct node_data *data = proxy->user_data;
+	struct node_data *data = object;
+	struct pw_proxy *proxy = (struct pw_proxy*)data->client_node;
 	struct mix *mix;
-	struct pw_memmap *mm;
+	struct pw_memmap *mm, *old;
 	void *ptr;
 	int res = 0;
 	uint32_t tag[5] = { data->remote_id, direction, port_id, mix_id, id };
@@ -749,11 +745,10 @@ client_node_port_set_io(void *object,
 	mix = ensure_mix(data, direction, port_id, mix_id);
 	if (mix == NULL) {
 		res = -ENOENT;
-		goto error_exit;
+		goto exit;
 	}
 
-	if ((mm = pw_mempool_find_tag(data->pool, tag, sizeof(tag))) != NULL)
-		pw_memmap_free(mm);
+	old = pw_mempool_find_tag(data->pool, tag, sizeof(tag));
 
 	if (memid == SPA_ID_INVALID) {
 		mm = ptr = NULL;
@@ -763,8 +758,9 @@ client_node_port_set_io(void *object,
 		mm = pw_mempool_map_id(data->pool, memid,
 				PW_MEMMAP_FLAG_READWRITE, offset, size, tag);
 		if (mm == NULL) {
+			pw_log_warn("can't map memory id %u: %m", memid);
 			res = -errno;
-			goto error_exit;
+			goto exit;
 		}
 		ptr = mm->ptr;
 	}
@@ -775,9 +771,6 @@ client_node_port_set_io(void *object,
 	if (id == SPA_IO_Buffers) {
 		if (ptr == NULL && mix->mix.io)
 			deactivate_mix(data, mix);
-		mix->mix.io = ptr;
-		if (ptr)
-			activate_mix(data, mix);
 	}
 
 	if ((res = spa_node_port_set_io(mix->port->mix,
@@ -785,29 +778,37 @@ client_node_port_set_io(void *object,
 		if (res == -ENOTSUP)
 			res = 0;
 		else
-			goto error_exit;
+			goto exit_free;
+	}
+	if (id == SPA_IO_Buffers) {
+		mix->mix.io = ptr;
+		if (ptr)
+			activate_mix(data, mix);
+	}
+exit_free:
+	if (old != NULL)
+		pw_memmap_free(old);
+exit:
+	if (res < 0) {
+		pw_log_error("port %p: set_io: %s", mix, spa_strerror(res));
+		pw_proxy_errorf(proxy, res, "port_set_io failed: %s", spa_strerror(res));
 	}
-	return res;
-
-error_exit:
-        pw_log_error("port %p: set_io: %s", mix, spa_strerror(res));
-	pw_proxy_errorf(proxy, res, "port_set_io failed: %s", spa_strerror(res));
 	return res;
 }
 
 static int link_signal_func(void *user_data)
 {
 	struct link *link = user_data;
-	uint64_t cmd = 1;
+	struct spa_system *data_system = link->data->context->data_system;
 	struct timespec ts;
 
 	pw_log_trace("link %p: signal", link);
 
-	clock_gettime(CLOCK_MONOTONIC, &ts);
+	spa_system_clock_gettime(data_system, CLOCK_MONOTONIC, &ts);
 	link->target.activation->status = PW_NODE_ACTIVATION_TRIGGERED;
 	link->target.activation->signal_time = SPA_TIMESPEC_TO_NSEC(&ts);
 
-	if (write(link->signalfd, &cmd, sizeof(cmd)) != sizeof(cmd))
+	if (SPA_UNLIKELY(spa_system_eventfd_write(data_system, link->signalfd, 1) < 0))
 		pw_log_warn("link %p: write failed %m", link);
 
 	return 0;
@@ -832,8 +833,8 @@ client_node_set_activation(void *object,
                         uint32_t offset,
                         uint32_t size)
 {
-	struct pw_proxy *proxy = object;
-	struct node_data *data = proxy->user_data;
+	struct node_data *data = object;
+	struct pw_proxy *proxy = (struct pw_proxy*)data->client_node;
 	struct pw_impl_node *node = data->node;
 	struct pw_memmap *mm;
 	void *ptr;
@@ -843,15 +844,14 @@ client_node_set_activation(void *object,
 	if (data->remote_id == node_id) {
 		pw_log_debug("node %p: our activation %u: %u %u %u", node, node_id,
 				memid, offset, size);
-		close(signalfd);
+		spa_system_close(data->context->data_system, signalfd);
 		return 0;
 	}
 
 	if (memid == SPA_ID_INVALID) {
 		mm = ptr = NULL;
 		size = 0;
-	}
-	else {
+	} else {
 		mm = pw_mempool_map_id(data->pool, memid,
 				PW_MEMMAP_FLAG_READWRITE, offset, size, NULL);
 		if (mm == NULL) {
@@ -862,7 +862,6 @@ client_node_set_activation(void *object,
 	}
 	pw_log_debug("node %p: set activation %d %p %u %u", node, node_id, ptr, offset, size);
 
-
 	if (ptr) {
 		link = calloc(1, sizeof(struct link));
 		if (link == NULL) {
@@ -919,22 +918,21 @@ static const struct pw_client_node_events client_node_events = {
 	.set_activation = client_node_set_activation,
 };
 
-static void do_node_init(struct pw_proxy *proxy)
+static void do_node_init(struct node_data *data)
 {
-	struct node_data *data = proxy->user_data;
 	struct pw_impl_port *port;
 
 	pw_log_debug("%p: init", data);
-	add_node_update(proxy, PW_CLIENT_NODE_UPDATE_PARAMS |
+	add_node_update(data, PW_CLIENT_NODE_UPDATE_PARAMS |
 				PW_CLIENT_NODE_UPDATE_INFO);
 
 	spa_list_for_each(port, &data->node->input_ports, link) {
-		add_port_update(proxy, port,
+		add_port_update(data, port,
 				PW_CLIENT_NODE_PORT_UPDATE_PARAMS |
 				PW_CLIENT_NODE_PORT_UPDATE_INFO);
 	}
 	spa_list_for_each(port, &data->node->output_ports, link) {
-		add_port_update(proxy, port,
+		add_port_update(data, port,
 				PW_CLIENT_NODE_PORT_UPDATE_PARAMS |
 				PW_CLIENT_NODE_PORT_UPDATE_INFO);
 	}
@@ -978,11 +976,8 @@ static void node_destroy(void *data)
 static void node_free(void *data)
 {
 	struct node_data *d = data;
-
 	pw_log_debug("%p: free", d);
-
-	if (d->client_node)
-		pw_proxy_destroy((struct pw_proxy*)d->client_node);
+	d->node = NULL;
 }
 
 static void node_info_changed(void *data, const struct pw_node_info *info)
@@ -992,6 +987,9 @@ static void node_info_changed(void *data, const struct pw_node_info *info)
 
 	pw_log_debug("info changed %p", d);
 
+	if (d->client_node == NULL)
+		return;
+
 	change_mask = 0;
 	if (info->change_mask & PW_NODE_CHANGE_MASK_PROPS)
 		change_mask |= PW_CLIENT_NODE_UPDATE_INFO;
@@ -999,7 +997,7 @@ static void node_info_changed(void *data, const struct pw_node_info *info)
 		change_mask |= PW_CLIENT_NODE_UPDATE_PARAMS;
 		change_mask |= PW_CLIENT_NODE_UPDATE_INFO;
 	}
-	add_node_update((struct pw_proxy*)d->client_node, change_mask);
+	add_node_update(d, change_mask);
 }
 
 static void node_port_info_changed(void *data, struct pw_impl_port *port,
@@ -1010,19 +1008,41 @@ static void node_port_info_changed(void *data, struct pw_impl_port *port,
 
 	pw_log_debug("info changed %p", d);
 
+	if (d->client_node == NULL)
+		return;
+
 	if (info->change_mask & PW_PORT_CHANGE_MASK_PROPS)
 		change_mask |= PW_CLIENT_NODE_PORT_UPDATE_INFO;
 	if (info->change_mask & PW_PORT_CHANGE_MASK_PARAMS) {
 		change_mask |= PW_CLIENT_NODE_PORT_UPDATE_PARAMS;
 		change_mask |= PW_CLIENT_NODE_PORT_UPDATE_INFO;
 	}
-	add_port_update((struct pw_proxy*)d->client_node, port, change_mask);
+	add_port_update(d, port, change_mask);
+}
+
+static void node_port_removed(void *data, struct pw_impl_port *port)
+{
+	struct node_data *d = data;
+
+	pw_log_debug("removed %p", d);
+
+	if (d->client_node == NULL)
+		return;
+
+	pw_client_node_port_update(d->client_node,
+			port->direction,
+			port->port_id,
+			0, 0, NULL, NULL);
 }
 
 static void node_active_changed(void *data, bool active)
 {
 	struct node_data *d = data;
 	pw_log_debug("active %d", active);
+
+	if (d->client_node == NULL)
+		return;
+
 	pw_client_node_set_active(d->client_node, active);
 }
 
@@ -1032,26 +1052,35 @@ static const struct pw_impl_node_events node_events = {
 	.free = node_free,
 	.info_changed = node_info_changed,
 	.port_info_changed = node_port_info_changed,
+	.port_removed = node_port_removed,
 	.active_changed = node_active_changed,
 };
 
+static void client_node_removed(void *_data)
+{
+	struct node_data *data = _data;
+	pw_log_debug("%p: removed", data);
+
+	spa_hook_remove(&data->proxy_client_node_listener);
+
+	if (data->node) {
+		spa_hook_remove(&data->node_listener);
+		pw_impl_node_set_state(data->node, PW_NODE_STATE_SUSPENDED);
+
+		clean_node(data);
+
+		if (data->do_free)
+			pw_impl_node_destroy(data->node);
+	}
+	data->client_node = NULL;
+}
+
 static void client_node_destroy(void *_data)
 {
 	struct node_data *data = _data;
 
 	pw_log_debug("%p: destroy", data);
-
-	clean_node(data);
-
-	spa_hook_remove(&data->node_listener);
-
-	data->client_node = NULL;
-
-	if (data->proxy)
-		pw_proxy_destroy(data->proxy);
-
-	if (data->do_free)
-		pw_impl_node_destroy(data->node);
+	client_node_removed(_data);
 }
 
 static void client_node_bound(void *_data, uint32_t global_id)
@@ -1063,50 +1092,33 @@ static void client_node_bound(void *_data, uint32_t global_id)
 
 static const struct pw_proxy_events proxy_client_node_events = {
 	PW_VERSION_PROXY_EVENTS,
+	.removed = client_node_removed,
 	.destroy = client_node_destroy,
 	.bound = client_node_bound,
 };
 
-static void proxy_destroy(void *_data)
-{
-	struct node_data *data = _data;
-
-	pw_log_debug("%p: destroy", data);
-	spa_hook_remove(&data->proxy_listener);
-	data->proxy = NULL;
-
-	if (data->client_node)
-		pw_proxy_destroy((struct pw_proxy*)data->client_node);
-}
-
-
-static const struct pw_proxy_events proxy_events = {
-	PW_VERSION_PROXY_EVENTS,
-	.destroy = proxy_destroy,
-};
-
 static int node_ready(void *d, int status)
 {
 	struct node_data *data = d;
 	struct pw_impl_node *node = data->node;
 	struct pw_node_activation *a = node->rt.activation;
+	struct spa_system *data_system = data->context->data_system;
 	struct timespec ts;
 	struct pw_impl_port *p;
-	uint64_t cmd = 1;
 
 	pw_log_trace("node %p: ready driver:%d exported:%d status:%d", node,
 			node->driver, node->exported, status);
 
-	if (status == SPA_STATUS_HAVE_DATA) {
+	if (status & SPA_STATUS_HAVE_DATA) {
 		spa_list_for_each(p, &node->rt.output_mix, rt.node_link)
 			spa_node_process(p->mix);
 	}
 
-	clock_gettime(CLOCK_MONOTONIC, &ts);
+	spa_system_clock_gettime(data_system, CLOCK_MONOTONIC, &ts);
 	a->status = PW_NODE_ACTIVATION_TRIGGERED;
 	a->signal_time = SPA_TIMESPEC_TO_NSEC(&ts);
 
-	if (write(data->rtwritefd, &cmd, sizeof(cmd)) != sizeof(cmd))
+	if (SPA_UNLIKELY(spa_system_eventfd_write(data_system, data->rtwritefd, 1) < 0))
 		pw_log_warn("node %p: write failed %m", node);
 
 	return 0;
@@ -1131,6 +1143,8 @@ static int node_xrun(void *d, uint64_t trigger, uint64_t delay, struct spa_pod *
 	pw_log_debug("node %p: XRun! count:%u time:%"PRIu64" delay:%"PRIu64" max:%"PRIu64,
 			node, a->xrun_count, trigger, delay, a->max_delay);
 
+	pw_context_driver_emit_xrun(data->context, node);
+
 	return 0;
 }
 
@@ -1150,16 +1164,19 @@ static struct pw_proxy *node_export(struct pw_core *core, void *object, bool do_
 	const char *str;
 	int i;
 
+	user_data_size = SPA_ROUND_UP_N(user_data_size, __alignof__(struct node_data));
+
 	client_node = pw_core_create_object(core,
 			"client-node",
 			PW_TYPE_INTERFACE_ClientNode,
 			PW_VERSION_CLIENT_NODE,
 			&node->properties->dict,
-			sizeof(struct node_data));
-        if (client_node == NULL)
-                return NULL;
+			user_data_size + sizeof(struct node_data));
+	if (client_node == NULL)
+		return NULL;
 
 	data = pw_proxy_get_user_data(client_node);
+	data = SPA_MEMBER(data, user_data_size, struct node_data);
 	data->pool = pw_core_get_mempool(core);
 	data->node = node;
 	data->do_free = do_free;
@@ -1192,18 +1209,13 @@ static struct pw_proxy *node_export(struct pw_core *core, void *object, bool do_
 	spa_node_set_callbacks(node->node, &node_callbacks, data);
 	pw_impl_node_add_listener(node, &data->node_listener, &node_events, data);
 
-        pw_client_node_add_listener(data->client_node,
+	pw_client_node_add_listener(data->client_node,
 					  &data->client_node_listener,
 					  &client_node_events,
-					  client_node);
-        do_node_init(client_node);
+					  data);
+	do_node_init(data);
 
-	data->proxy = (struct pw_proxy*) pw_client_node_get_node(data->client_node,
-			PW_VERSION_NODE, user_data_size);
-
-	pw_proxy_add_listener(data->proxy, &data->proxy_listener, &proxy_events, data);
-
-	return data->proxy;
+	return client_node;
 }
 
 struct pw_proxy *pw_core_node_export(struct pw_core *core,
