@@ -46,7 +46,6 @@
 
 #define MAX_SAMPLES	8192
 #define MAX_BUFFERS	64
-#define MIN_QUEUED	1
 
 #define MASK_BUFFERS	(MAX_BUFFERS-1)
 #define MAX_PORTS	1024
@@ -58,6 +57,7 @@ struct buffer {
 	uint32_t id;
 #define BUFFER_FLAG_MAPPED	(1 << 0)
 #define BUFFER_FLAG_QUEUED	(1 << 1)
+#define BUFFER_FLAG_ADDED	(1 << 2)
 	uint32_t flags;
 };
 
@@ -124,6 +124,10 @@ struct filter {
 	struct spa_callbacks callbacks;
 	struct spa_io_position *position;
 
+	struct {
+		struct spa_io_position *position;
+	} rt;
+
 	struct spa_list port_list;;
 	struct port *ports[2][MAX_PORTS];
 
@@ -137,7 +141,7 @@ struct filter {
 	struct pw_time time;
 
 	unsigned int disconnecting:1;
-	unsigned int free_proxy:1;
+	unsigned int disconnect_core:1;
 	unsigned int subscribe:1;
 	unsigned int draining:1;
 	unsigned int allow_mlock:1;
@@ -280,7 +284,7 @@ static inline struct buffer *pop_queue(struct port *port, struct queue *queue)
 	uint32_t index, id;
 	struct buffer *buffer;
 
-	if ((avail = spa_ringbuffer_get_read_index(&queue->ring, &index)) < MIN_QUEUED) {
+	if ((avail = spa_ringbuffer_get_read_index(&queue->ring, &index)) < 1) {
 		errno = EPIPE;
 		return NULL;
 	}
@@ -323,6 +327,15 @@ static bool filter_set_state(struct pw_filter *filter, enum pw_filter_state stat
 	return res;
 }
 
+static int
+do_set_position(struct spa_loop *loop,
+		bool async, uint32_t seq, const void *data, size_t size, void *user_data)
+{
+	struct filter *impl = user_data;
+	impl->rt.position = impl->position;
+	return 0;
+}
+
 static int impl_set_io(void *object, uint32_t id, void *data, size_t size)
 {
 	struct filter *impl = object;
@@ -335,6 +348,8 @@ static int impl_set_io(void *object, uint32_t id, void *data, size_t size)
 			impl->position = data;
 		else
 			impl->position = NULL;
+		pw_loop_invoke(impl->context->data_loop,
+			do_set_position, 1, NULL, 0, true, impl);
 		break;
 	}
 	pw_filter_emit_io_changed(&impl->this, NULL, id, data, size);
@@ -456,13 +471,14 @@ static int impl_port_enum_params(void *object, int seq,
 	struct spa_result_node_params result;
 	uint8_t buffer[1024];
 	struct spa_pod_builder b = { 0 };
-	uint32_t idx = 0, count = 0;
+	uint32_t count = 0;
 	struct param *p;
+	bool found = false;
 
 	spa_return_val_if_fail(num != 0, -EINVAL);
 
 	result.id = id;
-	result.next = start;
+	result.next = 0;
 
 	if ((port = get_port(d, direction, port_id)) == NULL)
 		return -EINVAL;
@@ -474,15 +490,16 @@ static int impl_port_enum_params(void *object, int seq,
 	spa_list_for_each(p, &port->param_list, link) {
 		struct spa_pod *param;
 
-		if (idx++ < start)
-			continue;
-
 		result.index = result.next++;
+		if (result.index < start)
+			continue;
 
 		param = p->param;
 		if (param == NULL || p->id != id)
 			continue;
 
+		found = true;
+
 		spa_pod_builder_init(&b, buffer, sizeof(buffer));
 		if (spa_pod_filter(&b, &result.param, param, filter) != 0)
 			continue;
@@ -492,7 +509,7 @@ static int impl_port_enum_params(void *object, int seq,
 		if (++count == num)
 			break;
 	}
-	return 0;
+	return found ? 0 : -ENOENT;
 }
 
 static int update_params(struct filter *impl, struct port *port, uint32_t id,
@@ -569,7 +586,8 @@ static void clear_buffers(struct port *port)
 	for (i = 0; i < port->n_buffers; i++) {
 		struct buffer *b = &port->buffers[i];
 
-		pw_filter_emit_remove_buffer(&impl->this, port->user_data, &b->this);
+		if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_ADDED))
+			pw_filter_emit_remove_buffer(&impl->this, port->user_data, &b->this);
 
 		if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_MAPPED)) {
 			for (j = 0; j < b->this.buffer->n_datas; j++) {
@@ -595,21 +613,21 @@ static int impl_port_set_param(void *object,
 	struct port *port;
 	int res;
 
-	if (impl->disconnecting)
-		return param == NULL ? 0 : -EIO;
+	if (impl->disconnecting && param != NULL)
+		return -EIO;
 
 	pw_log_debug(NAME" %p: param changed: %p %d", impl, param, impl->disconnecting);
 
 	if ((port = get_port(impl, direction, port_id)) == NULL)
 		return -EINVAL;
 
-	if (param && pw_log_level_enabled(SPA_LOG_LEVEL_DEBUG))
-		spa_debug_pod(2, NULL, param);
+	if (param)
+		pw_log_pod(SPA_LOG_LEVEL_DEBUG, param);
 
 	if ((res = update_params(impl, port, id, &param, param ? 1 : 0)) < 0)
 		return res;
 
-	if (id == SPA_PARAM_Format && param == NULL)
+	if (id == SPA_PARAM_Format)
 		clear_buffers(port);
 
 	pw_filter_emit_param_changed(filter, port->user_data, id, param);
@@ -617,10 +635,7 @@ static int impl_port_set_param(void *object,
 	if (filter->state == PW_FILTER_STATE_ERROR)
 		return -EIO;
 
-	if (port)
-		emit_port_info(impl, port, false);
-	else
-		emit_node_info(impl, false);
+	emit_port_info(impl, port, false);
 
 	return res;
 }
@@ -637,8 +652,8 @@ static int impl_port_use_buffers(void *object,
 	int prot, res;
 	int size = 0;
 
-	if (impl->disconnecting)
-		return n_buffers == 0 ? 0 : -EIO;
+	if (impl->disconnecting && n_buffers > 0)
+		return -EIO;
 
 	if ((port = get_port(impl, direction, port_id)) == NULL)
 		return -EINVAL;
@@ -662,6 +677,7 @@ static int impl_port_use_buffers(void *object,
 				    d->type == SPA_DATA_DmaBuf) {
 					if ((res = map_data(impl, d, prot)) < 0)
 						return res;
+					SPA_FLAG_SET(b->flags, BUFFER_FLAG_MAPPED);
 				}
 				else if (d->data == NULL) {
 					pw_log_error(NAME" %p: invalid buffer mem", filter);
@@ -669,7 +685,6 @@ static int impl_port_use_buffers(void *object,
 				}
 				buf_size += d->maxsize;
 			}
-			SPA_FLAG_SET(b->flags, BUFFER_FLAG_MAPPED);
 
 			if (size > 0 && buf_size != size) {
 				pw_log_error(NAME" %p: invalid buffer size %d", filter, buf_size);
@@ -684,8 +699,6 @@ static int impl_port_use_buffers(void *object,
 	for (i = 0; i < n_buffers; i++) {
 		struct buffer *b = &port->buffers[i];
 
-		b->flags = 0;
-		b->id = i;
 		b->this.buffer = buffers[i];
 
 		if (port->direction == SPA_DIRECTION_OUTPUT) {
@@ -693,6 +706,7 @@ static int impl_port_use_buffers(void *object,
 			push_queue(port, &port->dequeued, b);
 		}
 
+		SPA_FLAG_SET(b->flags, BUFFER_FLAG_ADDED);
 		pw_filter_emit_add_buffer(filter, port->user_data, &b->this);
 	}
 
@@ -718,7 +732,7 @@ static int impl_port_reuse_buffer(void *object, uint32_t port_id, uint32_t buffe
 
 static inline void copy_position(struct filter *impl)
 {
-	struct spa_io_position *p = impl->position;
+	struct spa_io_position *p = impl->rt.position;
 	if (p != NULL) {
 		SEQ_WRITE(impl->seq);
 		impl->time.now = p->clock.nsec;
@@ -742,9 +756,10 @@ do_call_process(struct spa_loop *loop,
 
 static void call_process(struct filter *impl)
 {
+	struct pw_filter *filter = &impl->this;
 	pw_log_trace(NAME" %p: call process", impl);
 	if (SPA_FLAG_IS_SET(impl->flags, PW_FILTER_FLAG_RT_PROCESS)) {
-		do_call_process(NULL, false, 1, NULL, 0, impl);
+		pw_filter_emit_process(filter, impl->rt.position);
 	}
 	else {
 		pw_loop_invoke(impl->context->main_loop,
@@ -777,7 +792,7 @@ static int impl_node_process(void *object)
 	struct buffer *b;
 	bool drained = true;
 
-	pw_log_trace(NAME" %p: do process %p", impl, impl->position);
+	pw_log_trace(NAME" %p: do process %p", impl, impl->rt.position);
 
 	/** first dequeue and recycle buffers */
 	spa_list_for_each(p, &impl->port_list, link) {
@@ -864,15 +879,22 @@ static const struct spa_node_methods impl_node = {
 	.process = impl_node_process,
 };
 
-static void proxy_destroy(void *_data)
+static void proxy_removed(void *_data)
 {
 	struct pw_filter *filter = _data;
-	filter->proxy = NULL;
+	pw_log_debug(NAME" %p: removed", filter);
 	spa_hook_remove(&filter->proxy_listener);
 	filter->node_id = SPA_ID_INVALID;
 	filter_set_state(filter, PW_FILTER_STATE_UNCONNECTED, NULL);
 }
 
+static void proxy_destroy(void *_data)
+{
+	struct pw_filter *filter = _data;
+	pw_log_debug(NAME" %p: destroy", filter);
+	proxy_removed(_data);
+}
+
 static void proxy_error(void *_data, int seq, int res, const char *message)
 {
 	struct pw_filter *filter = _data;
@@ -888,6 +910,7 @@ static void proxy_bound(void *_data, uint32_t global_id)
 
 static const struct pw_proxy_events proxy_events = {
 	PW_VERSION_PROXY_EVENTS,
+	.removed = proxy_removed,
 	.destroy = proxy_destroy,
 	.error = proxy_error,
 	.bound = proxy_bound,
@@ -897,9 +920,10 @@ static void on_core_error(void *_data, uint32_t id, int seq, int res, const char
 {
 	struct pw_filter *filter = _data;
 
-	pw_log_error(NAME" %p: error id:%u seq:%d res:%d (%s): %s", filter,
+	pw_log_debug(NAME" %p: error id:%u seq:%d res:%d (%s): %s", filter,
 			id, seq, res, spa_strerror(res), message);
-	if (id == 0) {
+
+	if (id == PW_ID_CORE && res == -EPIPE) {
 		filter_set_state(filter, PW_FILTER_STATE_UNCONNECTED, message);
 	}
 }
@@ -1014,11 +1038,14 @@ pw_filter_new_simple(struct pw_loop *loop,
 		return NULL;
 
 	context = pw_context_new(loop, NULL, 0);
-	if (context == NULL)
-		return NULL;
+	if (context == NULL) {
+		res = -errno;
+		goto error_cleanup;
+	}
 
 	impl = filter_new(context, name, props, props);
 	if (impl == NULL) {
+		props = NULL;
 		res = -errno;
 		goto error_cleanup;
 	}
@@ -1031,7 +1058,10 @@ pw_filter_new_simple(struct pw_loop *loop,
 	return this;
 
 error_cleanup:
-	pw_context_destroy(context);
+	if (context)
+		pw_context_destroy(context);
+	if (props)
+		pw_properties_free(props);
 	errno = -res;
 	return NULL;
 }
@@ -1058,6 +1088,7 @@ SPA_EXPORT
 void pw_filter_destroy(struct pw_filter *filter)
 {
 	struct filter *impl = SPA_CONTAINER_OF(filter, struct filter, this);
+	struct port *p;
 
 	pw_log_debug(NAME" %p: destroy", filter);
 
@@ -1065,6 +1096,9 @@ void pw_filter_destroy(struct pw_filter *filter)
 
 	pw_filter_disconnect(filter);
 
+	spa_list_consume(p, &impl->port_list, link)
+		pw_filter_remove_port(p->user_data);
+
 	if (filter->core) {
 		spa_hook_remove(&filter->core_listener);
 		spa_list_remove(&filter->link);
@@ -1212,7 +1246,7 @@ pw_filter_connect(struct pw_filter *filter,
 		spa_list_append(&filter->core->filter_list, &filter->link);
 		pw_core_add_listener(filter->core,
 				&filter->core_listener, &core_events, filter);
-		impl->free_proxy = true;
+		impl->disconnect_core = true;
 	}
 
 	pw_log_debug(NAME" %p: export node %p", filter, &impl->impl_node);
@@ -1249,11 +1283,12 @@ int pw_filter_disconnect(struct pw_filter *filter)
 	pw_log_debug(NAME" %p: disconnect", filter);
 	impl->disconnecting = true;
 
-	if (filter->proxy)
+	if (filter->proxy) {
 		pw_proxy_destroy(filter->proxy);
-
-	if (impl->free_proxy) {
-		impl->free_proxy = false;
+		filter->proxy = NULL;
+	}
+	if (impl->disconnect_core) {
+		impl->disconnect_core = false;
 		spa_hook_remove(&filter->core_listener);
 		spa_list_remove(&filter->link);
 		pw_core_disconnect(filter->core);
@@ -1426,16 +1461,18 @@ int pw_filter_set_error(struct pw_filter *filter,
 	if (res < 0) {
 		va_list args;
 		char *value;
+		int r;
 
 		va_start(args, error);
-		if (vasprintf(&value, error, args) < 0)
+		r = vasprintf(&value, error, args);
+		va_end(args);
+		if (r <  0)
 			return -errno;
 
 		if (filter->proxy)
 			pw_proxy_error(filter->proxy, res, value);
+		filter_set_state(filter, PW_FILTER_STATE_ERROR, value);
 
-		filter_set_state(filter, PW_STREAM_STATE_ERROR, value);
-		va_end(args);
 		free(value);
 	}
 	return res;
