@@ -45,7 +45,6 @@
 #define NAME "stream"
 
 #define MAX_BUFFERS	64
-#define MIN_QUEUED	1
 
 #define MASK_BUFFERS	(MAX_BUFFERS-1)
 #define MAX_PORTS	1
@@ -55,6 +54,7 @@ struct buffer {
 	uint32_t id;
 #define BUFFER_FLAG_MAPPED	(1 << 0)
 #define BUFFER_FLAG_QUEUED	(1 << 1)
+#define BUFFER_FLAG_ADDED	(1 << 2)
 	uint32_t flags;
 };
 
@@ -94,6 +94,7 @@ struct stream {
 	const char *path;
 
 	struct pw_context *context;
+	struct spa_hook context_listener;
 
 	enum spa_direction direction;
 	enum pw_stream_flags flags;
@@ -104,8 +105,12 @@ struct stream {
 	struct spa_node_methods node_methods;
 	struct spa_hook_list hooks;
 	struct spa_callbacks callbacks;
-	struct spa_io_buffers *io;
+
 	struct spa_io_position *position;
+	struct spa_io_buffers *io;
+	struct {
+		struct spa_io_position *position;
+	} rt;
 
 	uint32_t change_mask_all;
 	struct spa_port_info port_info;
@@ -127,8 +132,9 @@ struct stream {
 	struct pw_time time;
 
 	unsigned int disconnecting:1;
-	unsigned int free_proxy:1;
+	unsigned int disconnect_core:1;
 	unsigned int draining:1;
+	unsigned int drained:1;
 	unsigned int allow_mlock:1;
 	unsigned int warn_mlock:1;
 };
@@ -179,6 +185,7 @@ static struct param *add_param(struct stream *impl,
 	idx = get_param_index(id);
 	if (idx != -1) {
 		impl->port_info.change_mask |= SPA_PORT_CHANGE_MASK_PARAMS;
+		impl->params[idx].flags ^= SPA_PARAM_INFO_SERIAL;
 		impl->params[idx].flags |= SPA_PARAM_INFO_READ;
 	}
 
@@ -246,7 +253,7 @@ static inline struct buffer *pop_queue(struct stream *stream, struct queue *queu
 	uint32_t index, id;
 	struct buffer *buffer;
 
-	if ((avail = spa_ringbuffer_get_read_index(&queue->ring, &index)) < MIN_QUEUED) {
+	if ((avail = spa_ringbuffer_get_read_index(&queue->ring, &index)) < 1) {
 		errno = EPIPE;
 		return NULL;
 	}
@@ -311,9 +318,10 @@ do_call_process(struct spa_loop *loop,
 
 static void call_process(struct stream *impl)
 {
+	struct pw_stream *stream = &impl->this;
 	pw_log_trace(NAME" %p: call process", impl);
 	if (SPA_FLAG_IS_SET(impl->flags, PW_STREAM_FLAG_RT_PROCESS)) {
-		do_call_process(NULL, false, 1, NULL, 0, impl);
+		pw_stream_emit_process(stream);
 	}
 	else {
 		pw_loop_invoke(impl->context->main_loop,
@@ -329,7 +337,6 @@ do_call_drained(struct spa_loop *loop,
 	struct pw_stream *stream = &impl->this;
 	pw_log_trace(NAME" %p: drained", stream);
 	pw_stream_emit_drained(stream);
-	impl->draining = false;
 	return 0;
 }
 
@@ -339,6 +346,15 @@ static void call_drained(struct stream *impl)
 		do_call_drained, 1, NULL, 0, false, impl);
 }
 
+static int
+do_set_position(struct spa_loop *loop,
+		bool async, uint32_t seq, const void *data, size_t size, void *user_data)
+{
+	struct stream *impl = user_data;
+	impl->rt.position = impl->position;
+	return 0;
+}
+
 static int impl_set_io(void *object, uint32_t id, void *data, size_t size)
 {
 	struct stream *impl = object;
@@ -353,6 +369,8 @@ static int impl_set_io(void *object, uint32_t id, void *data, size_t size)
 			impl->position = data;
 		else
 			impl->position = NULL;
+		pw_loop_invoke(impl->context->data_loop,
+				do_set_position, 1, NULL, 0, true, impl);
 		break;
 	}
 	pw_stream_emit_io_changed(stream, id, data, size);
@@ -480,13 +498,14 @@ static int impl_port_enum_params(void *object, int seq,
 	struct spa_result_node_params result;
 	uint8_t buffer[1024];
 	struct spa_pod_builder b = { 0 };
-	uint32_t idx = 0, count = 0;
+	uint32_t count = 0;
 	struct param *p;
+	bool found = false;
 
 	spa_return_val_if_fail(num != 0, -EINVAL);
 
 	result.id = id;
-	result.next = start;
+	result.next = 0;
 
 	pw_log_debug(NAME" %p: param id %d (%s) start:%d num:%d", d, id,
 			spa_debug_type_find_name(spa_type_param, id),
@@ -495,15 +514,16 @@ static int impl_port_enum_params(void *object, int seq,
 	spa_list_for_each(p, &d->param_list, link) {
 		struct spa_pod *param;
 
-		if (idx++ < start)
-			continue;
-
 		result.index = result.next++;
+		if (result.index < start)
+			continue;
 
 		param = p->param;
 		if (param == NULL || p->id != id)
 			continue;
 
+		found = true;
+
 		spa_pod_builder_init(&b, buffer, sizeof(buffer));
 		if (spa_pod_filter(&b, &result.param, param, filter) != 0)
 			continue;
@@ -513,7 +533,7 @@ static int impl_port_enum_params(void *object, int seq,
 		if (++count == num)
 			break;
 	}
-	return 0;
+	return found ? 0 : -ENOENT;
 }
 
 static int map_data(struct stream *impl, struct spa_data *data, int prot)
@@ -567,7 +587,8 @@ static void clear_buffers(struct pw_stream *stream)
 	for (i = 0; i < impl->n_buffers; i++) {
 		struct buffer *b = &impl->buffers[i];
 
-		pw_stream_emit_remove_buffer(stream, &b->this);
+		if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_ADDED))
+			pw_stream_emit_remove_buffer(stream, &b->this);
 
 		if (SPA_FLAG_IS_SET(b->flags, BUFFER_FLAG_MAPPED)) {
 			for (j = 0; j < b->this.buffer->n_datas; j++) {
@@ -592,19 +613,19 @@ static int impl_port_set_param(void *object,
 	struct pw_stream *stream = &impl->this;
 	int res;
 
-	if (impl->disconnecting)
-		return param == NULL ? 0 : -EIO;
+	if (impl->disconnecting && param != NULL)
+		return -EIO;
 
 	pw_log_debug(NAME" %p: param id %d (%s) changed: %p", impl, id,
 			spa_debug_type_find_name(spa_type_param, id), param);
 
-	if (param && pw_log_level_enabled(SPA_LOG_LEVEL_DEBUG))
-		spa_debug_pod(2, NULL, param);
+	if (param)
+		pw_log_pod(SPA_LOG_LEVEL_DEBUG, param);
 
 	if ((res = update_params(impl, id, &param, param ? 1 : 0)) < 0)
 		return res;
 
-	if (id == SPA_PARAM_Format && param == NULL)
+	if (id == SPA_PARAM_Format)
 		clear_buffers(stream);
 
 	pw_stream_emit_param_changed(stream, id, param);
@@ -628,8 +649,8 @@ static int impl_port_use_buffers(void *object,
 	int prot, res;
 	int size = 0;
 
-	if (impl->disconnecting)
-		return n_buffers == 0 ? 0 : -EIO;
+	if (impl->disconnecting && n_buffers > 0)
+		return -EIO;
 
 	prot = PROT_READ | (direction == SPA_DIRECTION_OUTPUT ? PROT_WRITE : 0);
 
@@ -649,6 +670,7 @@ static int impl_port_use_buffers(void *object,
 				    d->type == SPA_DATA_DmaBuf) {
 					if ((res = map_data(impl, d, prot)) < 0)
 						return res;
+					SPA_FLAG_SET(b->flags, BUFFER_FLAG_MAPPED);
 				}
 				else if (d->data == NULL) {
 					pw_log_error(NAME" %p: invalid buffer mem", stream);
@@ -656,7 +678,6 @@ static int impl_port_use_buffers(void *object,
 				}
 				buf_size += d->maxsize;
 			}
-			SPA_FLAG_SET(b->flags, BUFFER_FLAG_MAPPED);
 
 			if (size > 0 && buf_size != size) {
 				pw_log_error(NAME" %p: invalid buffer size %d", stream, buf_size);
@@ -671,8 +692,6 @@ static int impl_port_use_buffers(void *object,
 	for (i = 0; i < n_buffers; i++) {
 		struct buffer *b = &impl->buffers[i];
 
-		b->flags = 0;
-		b->id = i;
 		b->this.buffer = buffers[i];
 
 		if (impl->direction == SPA_DIRECTION_OUTPUT) {
@@ -680,6 +699,7 @@ static int impl_port_use_buffers(void *object,
 			push_queue(impl, &impl->dequeued, b);
 		}
 
+		SPA_FLAG_SET(b->flags, BUFFER_FLAG_ADDED);
 		pw_stream_emit_add_buffer(stream, &b->this);
 	}
 
@@ -699,7 +719,7 @@ static int impl_port_reuse_buffer(void *object, uint32_t port_id, uint32_t buffe
 
 static inline void copy_position(struct stream *impl, int64_t queued)
 {
-	struct spa_io_position *p = impl->position;
+	struct spa_io_position *p = impl->rt.position;
 	if (p != NULL) {
 		SEQ_WRITE(impl->seq);
 		impl->time.now = p->clock.nsec;
@@ -717,12 +737,9 @@ static int impl_node_process_input(void *object)
 	struct pw_stream *stream = &impl->this;
 	struct spa_io_buffers *io = impl->io;
 	struct buffer *b;
-	uint64_t size;
 
-	size = impl->time.ticks - impl->dequeued.incount;
-
-	pw_log_trace(NAME" %p: process in status:%d id:%d ticks:%"PRIu64" delay:%"PRIi64" size:%"PRIi64,
-			stream, io->status, io->buffer_id, impl->time.ticks, impl->time.delay, size);
+	pw_log_trace(NAME" %p: process in status:%d id:%d ticks:%"PRIu64" delay:%"PRIi64,
+			stream, io->status, io->buffer_id, impl->time.ticks, impl->time.delay);
 
 	if (io->status != SPA_STATUS_HAVE_DATA)
 		goto done;
@@ -730,8 +747,6 @@ static int impl_node_process_input(void *object)
 	if ((b = get_buffer(stream, io->buffer_id)) == NULL)
 		goto done;
 
-	b->this.size = size;
-
 	/* push new buffer */
 	if (push_queue(impl, &impl->dequeued, b) == 0)
 		call_process(impl);
@@ -760,11 +775,9 @@ static int impl_node_process_output(void *object)
 	uint32_t index;
 
 again:
-	pw_log_trace(NAME" %p: process out status:%d id:%d ticks:%"PRIu64" delay:%"PRIi64, stream,
-			io->status, io->buffer_id, impl->time.ticks, impl->time.delay);
+	pw_log_trace(NAME" %p: process out status:%d id:%d", stream, io->status, io->buffer_id);
 
-	res = 0;
-	if (io->status != SPA_STATUS_HAVE_DATA) {
+	if ((res = io->status) != SPA_STATUS_HAVE_DATA) {
 		/* recycle old buffer */
 		if ((b = get_buffer(stream, io->buffer_id)) != NULL) {
 			pw_log_trace(NAME" %p: recycle buffer %d", stream, b->id);
@@ -774,31 +787,30 @@ again:
 		/* pop new buffer */
 		if ((b = pop_queue(impl, &impl->queued)) != NULL) {
 			io->buffer_id = b->id;
-			io->status = SPA_STATUS_HAVE_DATA;
+			res = io->status = SPA_STATUS_HAVE_DATA;
 			pw_log_trace(NAME" %p: pop %d %p", stream, b->id, io);
+		} else if (impl->draining) {
+			impl->drained = true;
+			io->buffer_id = SPA_ID_INVALID;
+			res = io->status = SPA_STATUS_DRAINED;
+			pw_log_trace(NAME" %p: draining", stream);
 		} else {
 			io->buffer_id = SPA_ID_INVALID;
-			io->status = SPA_STATUS_NEED_DATA;
+			res = io->status = SPA_STATUS_NEED_DATA;
 			pw_log_trace(NAME" %p: no more buffers %p", stream, io);
-			if (impl->draining) {
-				call_drained(impl);
-				goto exit;
-			}
 		}
 	}
 
 	if (!impl->draining &&
 	    !SPA_FLAG_IS_SET(impl->flags, PW_STREAM_FLAG_DRIVER) &&
-	    spa_ringbuffer_get_read_index(&impl->queued.ring, &index) < MIN_QUEUED) {
+	    spa_ringbuffer_get_read_index(&impl->dequeued.ring, &index) > 0) {
 		call_process(impl);
 		if (spa_ringbuffer_get_read_index(&impl->queued.ring, &index) > 0 &&
 		    io->status == SPA_STATUS_NEED_DATA)
 			goto again;
 	}
-exit:
 	copy_position(impl, impl->queued.outcount);
 
-	res = io->status;
 	pw_log_trace(NAME" %p: res %d", stream, res);
 
 	return res;
@@ -817,15 +829,22 @@ static const struct spa_node_methods impl_node = {
 	.port_reuse_buffer = impl_port_reuse_buffer,
 };
 
-static void proxy_destroy(void *_data)
+static void proxy_removed(void *_data)
 {
 	struct pw_stream *stream = _data;
-	stream->proxy = NULL;
+	pw_log_debug(NAME" %p: removed", stream);
 	spa_hook_remove(&stream->proxy_listener);
 	stream->node_id = SPA_ID_INVALID;
 	stream_set_state(stream, PW_STREAM_STATE_UNCONNECTED, NULL);
 }
 
+static void proxy_destroy(void *_data)
+{
+	struct pw_stream *stream = _data;
+	pw_log_debug(NAME" %p: destroy", stream);
+	proxy_removed(_data);
+}
+
 static void proxy_error(void *_data, int seq, int res, const char *message)
 {
 	struct pw_stream *stream = _data;
@@ -841,6 +860,7 @@ static void proxy_bound(void *data, uint32_t global_id)
 
 static const struct pw_proxy_events proxy_events = {
 	PW_VERSION_PROXY_EVENTS,
+	.removed = proxy_removed,
 	.destroy = proxy_destroy,
 	.error = proxy_error,
 	.bound = proxy_bound,
@@ -1027,9 +1047,10 @@ static void on_core_error(void *object, uint32_t id, int seq, int res, const cha
 {
 	struct pw_stream *stream = object;
 
-	pw_log_error(NAME" %p: error id:%u seq:%d res:%d (%s): %s", stream,
+	pw_log_debug(NAME" %p: error id:%u seq:%d res:%d (%s): %s", stream,
 			id, seq, res, spa_strerror(res), message);
-	if (id == 0) {
+
+	if (id == PW_ID_CORE && res == -EPIPE) {
 		stream_set_state(stream, PW_STREAM_STATE_UNCONNECTED, message);
 	}
 }
@@ -1039,6 +1060,20 @@ static const struct pw_core_events core_events = {
 	.error = on_core_error,
 };
 
+static void context_drained(void *data, struct pw_impl_node *node)
+{
+	struct stream *impl = data;
+	if (impl->node != node)
+		return;
+	if (impl->draining && impl->drained)
+		call_drained(impl);
+}
+
+static const struct pw_context_driver_events context_events = {
+	PW_VERSION_CONTEXT_DRIVER_EVENTS,
+	.drained = context_drained,
+};
+
 static struct stream *
 stream_new(struct pw_context *context, const char *name,
 		struct pw_properties *props, const struct pw_properties *extra)
@@ -1102,6 +1137,9 @@ stream_new(struct pw_context *context, const char *name,
 	impl->context = context;
 	impl->allow_mlock = context->defaults.mem_allow_mlock;
 
+	spa_hook_list_append(&impl->context->driver_listener_list,
+			&impl->context_listener,
+			&context_events, impl);
 	return impl;
 
 error_properties:
@@ -1155,12 +1193,15 @@ pw_stream_new_simple(struct pw_loop *loop,
 		return NULL;
 
 	context = pw_context_new(loop, NULL, 0);
-	if (context == NULL)
-		return NULL;
+	if (context == NULL) {
+		res = -errno;
+		goto error_cleanup;
+	}
 
 	impl = stream_new(context, name, props, NULL);
 	if (impl == NULL) {
 		res = -errno;
+		props = NULL;
 		goto error_cleanup;
 	}
 
@@ -1171,7 +1212,10 @@ pw_stream_new_simple(struct pw_loop *loop,
 	return this;
 
 error_cleanup:
-	pw_context_destroy(context);
+	if (context)
+		pw_context_destroy(context);
+	if (props)
+		pw_properties_free(props);
 	errno = -res;
 	return NULL;
 }
@@ -1226,6 +1270,8 @@ void pw_stream_destroy(struct pw_stream *stream)
 		free(c);
 	}
 
+	spa_hook_remove(&impl->context_listener);
+
 	if (impl->data.context)
 		pw_context_destroy(impl->data.context);
 
@@ -1314,7 +1360,7 @@ static int find_format(struct stream *impl, enum pw_direction direction,
 				SPA_PARAM_EnumFormat, &state,
 				NULL, &format, &b)) != 1) {
 		pw_log_warn(NAME" %p: no format given", impl);
-		return -ENOENT;
+		return 0;
 	}
 
 	if ((res = spa_format_parse(format, media_type, media_subtype)) < 0)
@@ -1344,9 +1390,9 @@ static const char *get_media_class(struct stream *impl)
 		case SPA_MEDIA_SUBTYPE_midi:
 			return "Midi";
 		}
-		/* fallthrough */
-	default:
 		return "Data";
+	default:
+		return "Unknown";
 	}
 }
 
@@ -1361,7 +1407,7 @@ pw_stream_connect(struct pw_stream *stream,
 {
 	struct stream *impl = SPA_CONTAINER_OF(stream, struct stream, this);
 	struct pw_impl_factory *factory;
-	struct pw_properties *props;
+	struct pw_properties *props = NULL;
 	struct pw_impl_node *follower;
 	const char *str;
 	uint32_t i;
@@ -1429,9 +1475,10 @@ pw_stream_connect(struct pw_stream *stream,
 	pw_properties_set(stream->properties, "mem.warn-mlock", impl->warn_mlock ? "true" : "false");
 
 	if ((pw_properties_get(stream->properties, PW_KEY_MEDIA_CLASS) == NULL)) {
+		const char *media_type = pw_properties_get(stream->properties, PW_KEY_MEDIA_TYPE);
 		pw_properties_setf(stream->properties, PW_KEY_MEDIA_CLASS, "Stream/%s/%s",
 				direction == PW_DIRECTION_INPUT ? "Input" : "Output",
-				get_media_class(impl));
+				media_type ? media_type : get_media_class(impl));
 	}
 	if ((str = pw_properties_get(stream->properties, PW_KEY_FORMAT_DSP)) != NULL)
 		pw_properties_set(impl->port_props, PW_KEY_FORMAT_DSP, str);
@@ -1451,11 +1498,15 @@ pw_stream_connect(struct pw_stream *stream,
 		spa_list_append(&stream->core->stream_list, &stream->link);
 		pw_core_add_listener(stream->core,
 				&stream->core_listener, &core_events, stream);
-		impl->free_proxy = true;
+		impl->disconnect_core = true;
 	}
 
 	pw_log_debug(NAME" %p: creating node", stream);
 	props = pw_properties_copy(stream->properties);
+	if (props == NULL) {
+		res = -errno;
+		goto error_node;
+	}
 
 	if ((str = pw_properties_get(props, PW_KEY_STREAM_MONITOR)) &&
 	    pw_properties_parse_bool(str)) {
@@ -1485,12 +1536,15 @@ pw_stream_connect(struct pw_stream *stream,
 				PW_VERSION_NODE,
 				props,
 				0);
+		props = NULL;
 		if (impl->node == NULL) {
 			res = -errno;
 			goto error_node;
 		}
 	} else {
 		impl->node = follower;
+		pw_properties_free(props);
+		props = NULL;
 	}
 	if (!SPA_FLAG_IS_SET(impl->flags, PW_STREAM_FLAG_INACTIVE))
 		pw_impl_node_set_active(impl->node, true);
@@ -1505,19 +1559,23 @@ pw_stream_connect(struct pw_stream *stream,
 
 	pw_proxy_add_listener(stream->proxy, &stream->proxy_listener, &proxy_events, stream);
 
-
 	pw_impl_node_add_listener(impl->node, &stream->node_listener, &node_events, stream);
 
 	return 0;
 
 error_connect:
 	pw_log_error(NAME" %p: can't connect: %s", stream, spa_strerror(res));
-	return res;
+	goto exit_cleanup;
 error_node:
 	pw_log_error(NAME" %p: can't make node: %s", stream, spa_strerror(res));
-	return res;
+	goto exit_cleanup;
 error_proxy:
 	pw_log_error(NAME" %p: can't make proxy: %s", stream, spa_strerror(res));
+	goto exit_cleanup;
+
+exit_cleanup:
+	if (props)
+		pw_properties_free(props);
 	return res;
 }
 
@@ -1538,15 +1596,17 @@ int pw_stream_disconnect(struct pw_stream *stream)
 	if (impl->node)
 		pw_impl_node_set_active(impl->node, false);
 
-	if (stream->proxy)
+	if (stream->proxy) {
 		pw_proxy_destroy(stream->proxy);
+		stream->proxy = NULL;
+	}
 
 	if (impl->node) {
 		pw_impl_node_destroy(impl->node);
 		impl->node = NULL;
 	}
-	if (impl->free_proxy) {
-		impl->free_proxy = false;
+	if (impl->disconnect_core) {
+		impl->disconnect_core = false;
 		spa_hook_remove(&stream->core_listener);
 		spa_list_remove(&stream->link);
 		pw_core_disconnect(stream->core);
@@ -1562,16 +1622,18 @@ int pw_stream_set_error(struct pw_stream *stream,
 	if (res < 0) {
 		va_list args;
 		char *value;
+		int r;
 
 		va_start(args, error);
-		if (vasprintf(&value, error, args) < 0)
+		r = vasprintf(&value, error, args);
+		va_end(args);
+		if (r < 0)
 			return -errno;
 
 		if (stream->proxy)
 			pw_proxy_error(stream->proxy, res, value);
 		stream_set_state(stream, PW_STREAM_STATE_ERROR, value);
 
-		va_end(args);
 		free(value);
 	}
 	return res;
@@ -1605,7 +1667,7 @@ int pw_stream_set_control(struct pw_stream *stream, uint32_t id, uint32_t n_valu
 	struct spa_pod *pod;
 	struct control *c;
 
-        va_start(varargs, values);
+	va_start(varargs, values);
 
 	spa_pod_builder_push_object(&b, &f[0], SPA_TYPE_OBJECT_Props, SPA_PARAM_Props);
 	while (1) {
@@ -1639,6 +1701,8 @@ int pw_stream_set_control(struct pw_stream *stream, uint32_t id, uint32_t n_valu
 	}
 	pod = spa_pod_builder_pop(&b, &f[0]);
 
+	va_end(varargs);
+
 	pw_impl_node_set_param(impl->node, SPA_PARAM_Props, 0, pod);
 
 	return 0;
@@ -1771,7 +1835,9 @@ do_drain(struct spa_loop *loop,
                  bool async, uint32_t seq, const void *data, size_t size, void *user_data)
 {
 	struct stream *impl = user_data;
+	pw_log_trace(NAME" %p", impl);
 	impl->draining = true;
+	impl->drained = false;
 	return 0;
 }
 
