@@ -139,17 +139,15 @@ static int open_read(struct midi_file *mf, const char *filename, struct midi_fil
 	uint16_t i;
 	struct stat st;
 
-	if (stat(filename, &st) < 0) {
-		res = -errno;
-		goto exit;
-	}
-
-	mf->size = st.st_size;
-
 	if ((mf->fd = open(filename, O_RDONLY)) < 0) {
 		res = -errno;
 		goto exit;
 	}
+	if (fstat(mf->fd, &st) < 0) {
+		res = -errno;
+		goto exit_close;
+	}
+	mf->size = st.st_size;
 
 	mf->data = mmap(NULL, mf->size, PROT_READ, MAP_SHARED, mf->fd, 0);
 	if (mf->data == MAP_FAILED) {
@@ -190,33 +188,41 @@ exit:
 	return res ;
 }
 
+static inline int write_n(int fd, const void *buf, int count)
+{
+	return write(fd, buf, count) == (ssize_t)count ? count : -errno;
+}
+
 static inline int write_be16(int fd, uint16_t val)
 {
 	uint8_t buf[2] = { val >> 8, val };
-	return write(fd, buf, 2);
+	return write_n(fd, buf, 2);
 }
 
 static inline int write_be32(int fd, uint32_t val)
 {
 	uint8_t buf[4] = { val >> 24, val >> 16, val >> 8, val };
-	return write(fd, buf, 4);
+	return write_n(fd, buf, 4);
 }
 
+#define CHECK_RES(expr) if ((res = (expr)) < 0) return res
+
 static int write_headers(struct midi_file *mf)
 {
 	struct midi_track *tr = &mf->tracks[0];
+	int res;
 
 	lseek(mf->fd, 0, SEEK_SET);
 
 	mf->length = 6;
-	write(mf->fd, "MThd", 4);
-	write_be32(mf->fd, mf->length);
-	write_be16(mf->fd, mf->info.format);
-	write_be16(mf->fd, mf->info.ntracks);
-	write_be16(mf->fd, mf->info.division);
+	CHECK_RES(write_n(mf->fd, "MThd", 4));
+	CHECK_RES(write_be32(mf->fd, mf->length));
+	CHECK_RES(write_be16(mf->fd, mf->info.format));
+	CHECK_RES(write_be16(mf->fd, mf->info.ntracks));
+	CHECK_RES(write_be16(mf->fd, mf->info.division));
 
-	write(mf->fd, "MTrk", 4);
-	write_be32(mf->fd, tr->size);
+	CHECK_RES(write_n(mf->fd, "MTrk", 4));
+	CHECK_RES(write_be32(mf->fd, tr->size));
 
 	return 0;
 }
@@ -242,9 +248,7 @@ static int open_write(struct midi_file *mf, const char *filename, struct midi_fi
 	mf->tempo = DEFAULT_TEMPO;
 	mf->info = *info;
 
-	write_headers(mf);
-
-	return 0;
+	res = write_headers(mf);
 exit:
 	return res;
 }
@@ -279,13 +283,15 @@ exit_free:
 
 int midi_file_close(struct midi_file *mf)
 {
+	int res;
+
 	if (mf->mode == 1) {
 		munmap(mf->data, mf->size);
 	} else if (mf->mode == 2) {
 		uint8_t buf[4] = { 0x00, 0xff, 0x2f, 0x00 };
-		write(mf->fd, buf, 4);
+		CHECK_RES(write_n(mf->fd, buf, 4));
 		mf->tracks[0].size += 4;
-		write_headers(mf);
+		CHECK_RES(write_headers(mf));
 	} else
 		return -EINVAL;
 
@@ -306,8 +312,7 @@ static int peek_next(struct midi_file *mf, struct midi_event *ev)
 		if (found == NULL || tr->tick < found->tick)
 			found = tr;
 	}
-	if (found == NULL ||
-	    tr_avail(found) == 0)
+	if (found == NULL)
 		return 0;
 
 	ev->track = found->id;
@@ -413,6 +418,7 @@ static int write_varlen(struct midi_file *mf, struct midi_track *tr, uint32_t va
 {
 	uint64_t buffer;
 	uint8_t b;
+	int res;
 
 	buffer = value & 0x7f;
 	while ((value >>= 7)) {
@@ -421,7 +427,7 @@ static int write_varlen(struct midi_file *mf, struct midi_track *tr, uint32_t va
 	}
         do  {
 		b = buffer & 0xff;
-		write(mf->fd, &b, 1);
+		CHECK_RES(write_n(mf->fd, &b, 1));
 		tr->size++;
 		buffer >>= 8;
 	} while (b & 0x80);
@@ -433,6 +439,7 @@ int midi_file_write_event(struct midi_file *mf, const struct midi_event *event)
 {
 	struct midi_track *tr;
 	uint32_t tick;
+	int res;
 
 	spa_return_val_if_fail(event != NULL, -EINVAL);
 	spa_return_val_if_fail(mf != NULL, -EINVAL);
@@ -443,10 +450,10 @@ int midi_file_write_event(struct midi_file *mf, const struct midi_event *event)
 
 	tick = event->sec * (1000000.0 * mf->info.division) / (double)mf->tempo;
 
-	write_varlen(mf, tr, tick - tr->tick);
+	CHECK_RES(write_varlen(mf, tr, tick - tr->tick));
 	tr->tick = tick;
 
-	write(mf->fd, event->data, event->size);
+	CHECK_RES(write_n(mf->fd, event->data, event->size));
 	tr->size += event->size;
 
 	return 0;
@@ -588,109 +595,109 @@ static const char *controller_name(uint8_t ctrl)
 	return controller_names[ctrl];
 }
 
-static void dump_mem(const char *label, uint8_t *data, uint32_t size)
+static void dump_mem(FILE *out, const char *label, uint8_t *data, uint32_t size)
 {
-	printf("%s: ", label);
+	fprintf(out, "%s: ", label);
 	while (size--)
-		printf("%02x ", *data++);
+		fprintf(out, "%02x ", *data++);
 }
 
-int midi_file_dump_event(const struct midi_event *ev)
+int midi_file_dump_event(FILE *out, const struct midi_event *ev)
 {
-	printf("track:%2d sec:%f ", ev->track, ev->sec);
+	fprintf(out, "track:%2d sec:%f ", ev->track, ev->sec);
 
 	switch (ev->data[0]) {
 	case 0x80 ... 0x8f:
-		printf("Note Off   (channel %2d): note %3s%d, velocity %3d",
+		fprintf(out, "Note Off   (channel %2d): note %3s%d, velocity %3d",
 				(ev->data[0] & 0x0f) + 1,
 				note_names[ev->data[1] % 12], ev->data[1] / 12 -1,
 				ev->data[2]);
 		break;
 	case 0x90 ... 0x9f:
-		printf("Note On    (channel %2d): note %3s%d, velocity %3d",
+		fprintf(out, "Note On    (channel %2d): note %3s%d, velocity %3d",
 				(ev->data[0] & 0x0f) + 1,
 				note_names[ev->data[1] % 12], ev->data[1] / 12 -1,
 				ev->data[2]);
 		break;
 	case 0xa0 ... 0xaf:
-		printf("Aftertouch (channel %2d): note %3s%d, pressure %3d",
+		fprintf(out, "Aftertouch (channel %2d): note %3s%d, pressure %3d",
 				(ev->data[0] & 0x0f) + 1,
 				note_names[ev->data[1] % 12], ev->data[1] / 12 -1,
 				ev->data[2]);
 		break;
 	case 0xb0 ... 0xbf:
-		printf("Controller (channel %2d): controller %3d (%s), value %3d",
+		fprintf(out, "Controller (channel %2d): controller %3d (%s), value %3d",
 				(ev->data[0] & 0x0f) + 1, ev->data[1],
 				controller_name(ev->data[1]), ev->data[2]);
 		break;
 	case 0xc0 ... 0xcf:
-		printf("Program    (channel %2d): program %3d (%s)",
+		fprintf(out, "Program    (channel %2d): program %3d (%s)",
 				(ev->data[0] & 0x0f) + 1, ev->data[1],
 				program_names[ev->data[1]]);
 		break;
 	case 0xd0 ... 0xdf:
-		printf("Channel Pressure (channel %2d): pressure %3d",
+		fprintf(out, "Channel Pressure (channel %2d): pressure %3d",
 				(ev->data[0] & 0x0f) + 1, ev->data[1]);
 		break;
 	case 0xe0 ... 0xef:
-		printf("Pitch Bend (channel %2d): value %d", (ev->data[0] & 0x0f) + 1,
+		fprintf(out, "Pitch Bend (channel %2d): value %d", (ev->data[0] & 0x0f) + 1,
 				((int)ev->data[2] << 7 | ev->data[1]) - 0x2000);
 		break;
 	case 0xf0:
 	case 0xf7:
-		dump_mem("SysEx", ev->data, ev->size);
+		dump_mem(out, "SysEx", ev->data, ev->size);
 		break;
 	case 0xff:
-		printf("Meta: ");
+		fprintf(out, "Meta: ");
 		switch (ev->data[1]) {
 		case 0x00:
-			printf("Sequence Number %3d %3d", ev->data[3], ev->data[4]);
+			fprintf(out, "Sequence Number %3d %3d", ev->data[3], ev->data[4]);
 			break;
 		case 0x01 ... 0x09:
-			printf("%s: %s", event_names[ev->data[1] - 1], &ev->data[ev->meta.offset]);
+			fprintf(out, "%s: %s", event_names[ev->data[1] - 1], &ev->data[ev->meta.offset]);
 			break;
 		case 0x20:
-			printf("Channel Prefix: %03d", ev->data[3]);
+			fprintf(out, "Channel Prefix: %03d", ev->data[3]);
 			break;
 		case 0x21:
-			printf("Midi Port: %03d", ev->data[3]);
+			fprintf(out, "Midi Port: %03d", ev->data[3]);
 			break;
 		case 0x2f:
-			printf("End Of Track");
+			fprintf(out, "End Of Track");
 			break;
 		case 0x51:
-			printf("Tempo: %d microseconds per quarter note, %.2f BPM",
+			fprintf(out, "Tempo: %d microseconds per quarter note, %.2f BPM",
 					ev->meta.parsed.tempo.uspqn,
 					60000000.0 / (double)ev->meta.parsed.tempo.uspqn);
 			break;
 		case 0x54:
-			printf("SMPTE Offset: %s %02d:%02d:%02d:%02d.%03d",
+			fprintf(out, "SMPTE Offset: %s %02d:%02d:%02d:%02d.%03d",
 					smpte_rates[(ev->data[3] & 0x60) >> 5],
 					ev->data[3] & 0x1f, ev->data[4], ev->data[5],
 					ev->data[6], ev->data[7]);
 			break;
 		case 0x58:
-			printf("Time Signature: %d/%d, %d clocks per click, %d notated 32nd notes per quarter note",
+			fprintf(out, "Time Signature: %d/%d, %d clocks per click, %d notated 32nd notes per quarter note",
 				ev->data[3], (int)pow(2, ev->data[4]), ev->data[5], ev->data[6]);
 			break;
 		case 0x59:
 		{
 			int sf = ev->data[3];
-			printf("Key Signature: %d %s: %s", abs(sf),
+			fprintf(out, "Key Signature: %d %s: %s", abs(sf),
 					sf > 0 ? "sharps" : "flats",
 					ev->data[4] == 0 ?
-						major_keys[SPA_CLAMP(sf + 9, 0, 19)] :
-						minor_keys[SPA_CLAMP(sf + 9, 0, 19)]);
+						major_keys[SPA_CLAMP(sf + 9, 0, 18)] :
+						minor_keys[SPA_CLAMP(sf + 9, 0, 18)]);
 			break;
 		}
 		case 0x7f:
-			dump_mem("Sequencer", ev->data, ev->size);
+			dump_mem(out, "Sequencer", ev->data, ev->size);
 			break;
 		default:
-			dump_mem("Invalid", ev->data, ev->size);
+			dump_mem(out, "Invalid", ev->data, ev->size);
 		}
 		break;
 	}
-	printf("\n");
+	fprintf(out, "\n");
 	return 0;
 }
