@@ -486,8 +486,8 @@ static unsigned int find_channel(const char *name)
 	int i;
 
 	for (i = 0; spa_type_audio_channel[i].name; i++) {
-		if (strcmp(name, rindex(spa_type_audio_channel[i].name, ':')+1) == 0)
-			return i;
+		if (strcmp(name, spa_debug_type_short_name(spa_type_audio_channel[i].name)) == 0)
+			return spa_type_audio_channel[i].type;
 	}
 	return SPA_AUDIO_CHANNEL_UNKNOWN;
 }
@@ -513,10 +513,9 @@ static int parse_channelmap(const char *channel_map, struct channelmap *map)
 	map->n_channels = nch;
 	for (i = 0; i < map->n_channels; i++) {
 		int c = find_channel(ch[i]);
-		if (c == SPA_AUDIO_CHANNEL_UNKNOWN)
-			return -1;
 		map->channels[i] = c;
 	}
+	pw_free_strv(ch);
 	return 0;
 }
 
@@ -563,7 +562,7 @@ static void channelmap_print(struct channelmap *map)
 		const char *name = spa_debug_type_find_name(spa_type_audio_channel, map->channels[i]);
 		if (name == NULL)
 			name = ":UNK";
-		printf("%s%s", rindex(name, ':')+1, i + 1 < map->n_channels ? "," : "");
+		printf("%s%s", spa_debug_type_short_name(name), i + 1 < map->n_channels ? "," : "");
 	}
 }
 
@@ -863,32 +862,32 @@ enum {
 };
 
 static const struct option long_options[] = {
-	{"help",	no_argument,	   NULL, 'h'},
-	{"version",	no_argument,	   NULL, OPT_VERSION},
-	{"verbose",	no_argument,	   NULL, 'v'},
+	{ "help",		no_argument,	   NULL, 'h' },
+	{ "version",		no_argument,	   NULL, OPT_VERSION},
+	{ "verbose",		no_argument,	   NULL, 'v' },
 
-	{"record",	no_argument,	   NULL, 'r'},
-	{"playback",	no_argument,	   NULL, 's'},
-	{"midi",	no_argument,	   NULL, 'm'},
+	{ "record",		no_argument,	   NULL, 'r' },
+	{ "playback",		no_argument,	   NULL, 's' },
+	{ "midi",		no_argument,	   NULL, 'm' },
 
-	{"remote",	required_argument, NULL, 'R'},
+	{ "remote",		required_argument, NULL, 'R' },
 
-	{"media-type",		required_argument, NULL, OPT_MEDIA_TYPE },
-	{"media-category",	required_argument, NULL, OPT_MEDIA_CATEGORY },
-	{"media-role",		required_argument, NULL, OPT_MEDIA_ROLE },
-	{"target",		required_argument, NULL, OPT_TARGET },
-	{"latency",		required_argument, NULL, OPT_LATENCY },
+	{ "media-type",		required_argument, NULL, OPT_MEDIA_TYPE },
+	{ "media-category",	required_argument, NULL, OPT_MEDIA_CATEGORY },
+	{ "media-role",		required_argument, NULL, OPT_MEDIA_ROLE },
+	{ "target",		required_argument, NULL, OPT_TARGET },
+	{ "latency",		required_argument, NULL, OPT_LATENCY },
 
-	{"rate",		required_argument, NULL, OPT_RATE },
-	{"channels",		required_argument, NULL, OPT_CHANNELS },
-	{"channel-map",		required_argument, NULL, OPT_CHANNELMAP },
-	{"format",		required_argument, NULL, OPT_FORMAT },
-	{"volume",		required_argument, NULL, OPT_VOLUME },
-	{"quality",		required_argument, NULL, 'q' },
+	{ "rate",		required_argument, NULL, OPT_RATE },
+	{ "channels",		required_argument, NULL, OPT_CHANNELS },
+	{ "channel-map",	required_argument, NULL, OPT_CHANNELMAP },
+	{ "format",		required_argument, NULL, OPT_FORMAT },
+	{ "volume",		required_argument, NULL, OPT_VOLUME },
+	{ "quality",		required_argument, NULL, 'q' },
 
-	{"list-targets",	no_argument, NULL, OPT_LIST_TARGETS },
+	{ "list-targets",	no_argument, NULL, OPT_LIST_TARGETS },
 
-	{NULL, 0, NULL, 0 }
+	{ NULL, 0, NULL, 0 }
 };
 
 static void show_usage(const char *name, bool is_error)
@@ -902,6 +901,7 @@ static void show_usage(const char *name, bool is_error)
 	fprintf(fp,
              "  -h, --help                            Show this help\n"
              "      --version                         Show version\n"
+             "  -v, --verbose                         Enable verbose operations\n"
 	     "\n");
 
 	fprintf(fp,
@@ -945,10 +945,6 @@ static void show_usage(const char *name, bool is_error)
 		     "  -m, --midi                            Midi mode\n"
 		     "\n");
 	}
-
-        fprintf(fp,
-             "  -v, --verbose                         Enable verbose operations\n"
-	     "\n");
 }
 
 static int midi_play(struct data *d, void *src, unsigned int n_frames)
@@ -990,7 +986,7 @@ static int midi_play(struct data *d, void *src, unsigned int n_frames)
 		midi_file_read_event(d->midi.file, &ev);
 
 		if (d->verbose)
-			midi_file_dump_event(&ev);
+			midi_file_dump_event(stdout, &ev);
 
 		if (ev.data[0] == 0xff)
 			continue;
@@ -1030,7 +1026,7 @@ static int midi_record(struct data *d, void *src, unsigned int n_frames)
 		ev.size = SPA_POD_BODY_SIZE(&c->value);
 
 		if (d->verbose)
-			midi_file_dump_event(&ev);
+			midi_file_dump_event(stdout, &ev);
 
 		midi_file_write_event(d->midi.file, &ev);
 	}
@@ -1072,6 +1068,7 @@ static int setup_sndfile(struct data *data)
 	const char *s;
 	unsigned int nom = 0;
 
+	spa_zero(info);
 	/* for record, you fill in the info first */
 	if (data->mode == mode_record) {
 		if (data->format == NULL)
@@ -1080,6 +1077,8 @@ static int setup_sndfile(struct data *data)
 			data->channels = DEFAULT_CHANNELS;
 		if (data->rate == 0)
 			data->rate = DEFAULT_RATE;
+		if (data->channelmap.n_channels == 0)
+			channelmap_default(&data->channelmap, data->channels);
 
 		memset(&info, 0, sizeof(info));
 		info.samplerate = data->rate;
@@ -1237,7 +1236,13 @@ int main(int argc, char *argv[])
 		data.mode = mode_playback;
 	else if (!strcmp(prog, "pw-record"))
 		data.mode = mode_record;
-	else
+	else if (!strcmp(prog, "pw-midiplay")) {
+		data.mode = mode_playback;
+		data.is_midi = true;
+	} else if (!strcmp(prog, "pw-midirecord")) {
+		data.mode = mode_record;
+		data.is_midi = true;
+	} else
 		data.mode = mode_none;
 
 	/* negative means no volume adjustment */
@@ -1580,6 +1585,8 @@ error_connect_fail:
 	if (data.stream)
 		pw_stream_destroy(data.stream);
 error_no_stream:
+	if (data.registry)
+		pw_proxy_destroy((struct pw_proxy*)data.registry);
 error_no_registry:
 	pw_core_disconnect(data.core);
 error_ctx_connect_failed:
@@ -1595,6 +1602,7 @@ error_bad_file:
 		sf_close(data.file);
 	if (data.midi.file)
 		midi_file_close(data.midi.file);
+	pw_deinit();
 	return exit_code;
 
 error_usage:
